/*
*  Program name:             circle  ver. 1.0
*
*  Program function:         Generate JKG data of circle
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            May 10, 1991
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. center x
*                            2. center y
*                            3. radius
*                            4. start position (unit:degree)
*                            5. end   position (unit:degree)
*                            6. interval       (unit:degree)
*
*  Input :                   None
*
*  Output:                   JKG data (to stdout)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	x0, y0, rad, str, fin, itv, x, y, r, pi, sin(), cos(), atof() ;
	char	hed ;
/* ---- parameter set ------------------------------------------------------- */
	pi = asin(1.0) / 90.0 ;
/* ---- argument read ---- */
	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr,
			" USAGE: circle {x0 y0 rad start end interval}\n") ;
		return(0) ;
		}
	if (argc > 1) x0  = atof(argv[1]) ; else x0  =   0.0 ;
	if (argc > 2) y0  = atof(argv[2]) ; else y0  =   0.0 ;
	if (argc > 3) rad = atof(argv[3]) ; else rad =   1.0 ;
	if (argc > 4) str = atof(argv[4]) ; else str =   0.0 ;
	if (argc > 5) fin = atof(argv[5]) ; else fin = 360.0 ;
	if (argc > 6) itv = atof(argv[6]) ; else itv =   1.0 ;
/* ---- Circle Output ---- */
	hed = 'm' ;
	for (r = str; r <= fin; r = r + itv) {
		x = x0 + rad * cos(r * pi) ;
		y = y0 + rad * sin(r * pi) ;
		printf("%c %lf %lf\n", hed, x, y) ;
		hed = 'l' ;
		}
	return(0) ;
	}
