/*
*  Program name:             dlgout  ver. 2.1
*
*  Program function:         Make JKG data from DLG data
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*
*  Date of revision:         June 5, 1995  for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*                            May 20, 2004 Change Fp, Fc option by Y. Murata
*
*  Arguments:                First  DLG file name
*                            -arc   draw arcs
*                            -poly  draw areas
*                            -bnd   draw polygon boundaries
*                            -del   delete boundaries between same color/pattern
*                            -delf  delete boundaries & fault between same col/pat
*                            -sec   print JKG in second
*                            -cf    dsn color/pattern table file name
*                            -pf    dsn pen width/color table file name
*                            -lim   limitation file name
*                            -ids   draw ids
*                            -coast no coastlines
*
*  Input:                    none (program opens the DLG data base)
*
*  Output:                   JKG to stdout
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

#define	Max_Tbl		1025		/* Max of Color-Pattern Table	*/
#define	Max_Pen		999999		/* Max of Pen Table		*/
#define Max_Buff	200		/* Max of arc			*/
int	Max_ID=48 ;			/* Max of Arcs in a Poly	*/
#define	Pint		10000
#define	Z(a)	(a)/Pint][(a)%Pint
int	Max_Point=Pint ;		/* Max of Points		*/

/* DLG */
FILE	*DLG ;
char	Line[500] ;

/* Header */
int	Node_n, Poly_n, Arc_n, Ae ;
double	A[4] ;
char	C_NAME[20] ;

/* Node */
int	Node_id, Node_Arc_n, *Arc ;
double	Node_x, Node_y ;

/* Poly */
int	Poly_id, Poly_Arc_n, Poly_Don_n ;
int	*Poly_Geo ;
double	Poly_x, Poly_y ;

/* Arc */
int	Arc_id, Arc_S_Node, Arc_E_Node, Arc_XY_n, Arc_Attr ;
int	*Arc_L_Poly, *Arc_R_Poly, *Arc_Fault, *XY_Pos, *In ;
int	Arc_id_old ;
double	*Arc_X[Max_Buff], *Arc_Y[Max_Buff] ;

/* Table */
int	Color[Max_Tbl], Pattern[Max_Tbl], Pattern_Color[Max_Tbl],
	Pcolor[Max_Pen], Pwidth[Max_Pen], Color_old=0, Width_old=0 ;

/* Width:  -1=no change, 0=no write, >0=width change	*/

int	Limit, To_Second, Coast ;
double	Xmin, Ymin, Xmax, Ymax ;
char	CR ;
int	Write_IDS ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	Write_Point, Write_Arc, Write_Poly, Write_Boundary, Write_Same ;
	int	i ;
	void	DLG_Head_Read(), DLG_Node_Read(), DLG_Poly_Read(), Limit_Set(),
		DLG_Arc_Read(), Poly_Write(), Color_Set(), Pen_Set() ;
	double	x, y ;
/* --------------------------------------------- File open ----------------- */
	if (argc < 2 || strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr, " USAGE: dlgout DLG-file {-point -arc -poly ") ;
	  fprintf(stderr, "-bnd -del -delf -sec -cf $ -pf $ ") ;
	  fprintf(stderr, "-lim $ -coast}\n") ;
	  return(0) ;
	  }
	if ((DLG = fopen(argv[1], "r")) == NULL) {
	  fprintf(stderr," Can't open DLG file '%s'.\n", argv[1]) ;
	  return(0) ;
	  }
/* --------------------------------------------- Parameter set ------------- */
	Write_Point = 0 ;
	Write_Arc = 0 ;
	Write_Poly = 0 ;
	Write_Boundary = 0 ;
	Write_Same = 1 ;
	Write_IDS = 0 ;
	Limit = 0 ;
	To_Second = 0 ;
	Coast = 0 ;
	CR = '\n' ;
	for (i = 0; i < Max_Tbl; i++) {
	  Color[i] = i ;
	  Pattern[i] = 0 ;
	  }
	for (i = 0; i < Max_Pen; i++) {
	  Pcolor[i] =  0 ;
	  Pwidth[i] = -1 ;
	  }
	for (i = 0; i < argc; i++) {
	       if (strcmp(argv[i], "-point")== 0) Write_Point= 1 ;
	  else if (strcmp(argv[i], "-arc")  == 0) Write_Arc  = 1 ;
	  else if (strcmp(argv[i], "-poly") == 0) Write_Poly = 1 ;
	  else if (strcmp(argv[i], "-bnd")  == 0) Write_Boundary = 1 ;
	  else if (strcmp(argv[i], "-del")  == 0) Write_Same = 0 ;
	  else if (strcmp(argv[i], "-delf") == 0) Write_Same = -1 ;
	  else if (strcmp(argv[i], "-cf")   == 0) Color_Set(argv[++i]) ;
	  else if (strcmp(argv[i], "-pf")   == 0) Pen_Set(argv[++i]) ;
	  else if (strcmp(argv[i], "-lim")  == 0) Limit_Set(argv[++i]) ;
	  else if (strcmp(argv[i], "-ids")  == 0) Write_IDS = 1 ;
	  else if (strcmp(argv[i], "-coast")  == 0) Coast = 1 ;
	  }
/* --------------------------------------------- Header read --------------- */
	fprintf(stderr, " DLG Header read\n") ;
	DLG_Head_Read() ;
/* --------------------------------------------- Memory alloc -------------- */
	if ((Poly_Geo = (int *)calloc(Poly_n+1, sizeof(*Poly_Geo))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Poly_Geo.\n") ;
	  return(-1) ;
	  }
	Poly_Geo[0] = 0 ;
	if ((Arc_L_Poly = (int *)calloc(Arc_n, sizeof(*Arc_L_Poly))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Arc_L_Poly.\n") ;
	  return(-1) ;
	  }
	if ((Arc_R_Poly = (int *)calloc(Arc_n, sizeof(*Arc_R_Poly))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Arc_R_Poly.\n") ;
	  return(-1) ;
	  }
	if ((Arc_Fault = (int *)calloc(Arc_n, sizeof(*Arc_Fault))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Arc_Fault.\n") ;
	  return(-1) ;
	  }
	if ((XY_Pos = (int *)calloc(Arc_n, sizeof(*XY_Pos))) == NULL) {
	  fprintf(stderr, " Can't alloc memory XY_Pos.\n") ;
	  return(-1) ;
	  }
	if ((In = (int *)calloc(Arc_n, sizeof(*In))) == NULL) {
	  fprintf(stderr, " Can't alloc memory In.\n") ;
	  return(-1) ;
	  }
	if ((Arc = (int *)calloc(Max_ID, sizeof(*Arc))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Arc.\n") ;
	  return(-1) ;
	  }
	if ((Arc_X[0] = (double *)calloc(Pint, sizeof(*Arc_X[0]))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Arc_X.\n") ;
	  return(-1) ;
	  }
	if ((Arc_Y[0] = (double *)calloc(Pint, sizeof(*Arc_Y[0]))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Arc_Y.\n") ;
	  return(-1) ;
	  }
/* --------------------------------------------- Node part read ------------ */
	fprintf(stderr, " DLG Node read\n") ;
	for (i = 0; i < Node_n; i++) DLG_Node_Read() ;
/* --------------------------------------------- Poly part read ------------ */
	fprintf(stderr, " DLG Poly read\n") ;
	for (i = 0; i < Poly_n; i++) {
	  DLG_Poly_Read() ;
	  if (Write_Point == 1) {
	    x = A[0] * Poly_x + A[1] * Poly_y + A[2] ;
	    y = A[0] * Poly_y - A[1] * Poly_x + A[3] ;
	    if (To_Second == 1) {
	      x = x * 3600.0 ;
	      y = y * 3600.0 ;
	      }
	    }
	  }
	if (Write_Point == 1) {
	  fclose(DLG) ;
	  return(-1) ;
	  }
/* --------------------------------------------- Arc part read ------------- */
	fprintf(stderr, " DLG Arc read %d\n", Arc_n) ;
	for (i = 0; i < Arc_n; i++) DLG_Arc_Read(Write_Arc, Write_Same) ;
	fprintf(stderr, " Total Point = %d\n", XY_Pos[Arc_id]) ;
/* --------------------------------------------- Poly part write ----------- */
	if (Write_Poly == 1) {
	  fprintf(stderr, " DLG Poly Write\n") ;
	  fseek(DLG, 0L, 0L) ;
	  DLG_Head_Read() ;
	  for (i = 0; i < Node_n; i++) DLG_Node_Read() ;
	  for (i = 0; i < Poly_n; i++) {
	    DLG_Poly_Read() ;
	    if (Poly_Arc_n > 0) Poly_Write(Write_Boundary, Write_Same) ;
	    }
	  }
/* --------------------------------------------- File close ---------------- */
	fclose(DLG) ;
/* ------------------------------------------------------------------------- */
	return(0) ;
	}
/* --------------------------------------------- Color_Table_Read ---------- */
void	Color_Set(dsn)
char	dsn[] ;
{
	FILE	*fp ;
	int	i, j, k, l ;

	if ((fp = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open color table file '%s'.\n", dsn) ;
	  exit(-1) ;
	  }
	while (fscanf(fp, "%d %d %d %d", &i, &j, &k, &l) == 4) {
	  if (i >= Max_Tbl || i < 0) {
	    fprintf(stderr, " Color Table Id >= Max_Tbl.\n") ;
	    exit(-1) ;
	    }
	  Color[i] = j ;
	  Pattern[i] = k ;
	  Pattern_Color[i] = l ;
	  }
	fclose(fp) ;
	}
/* --------------------------------------------- Pen_Table_Read ------------ */
void	Pen_Set(dsn)
char	dsn[] ;
{
	FILE	*fp ;
	int	i, j, k ;

	if ((fp = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open pen table file '%s'.\n", dsn) ;
	  exit(-1) ;
	  }
	while (fscanf(fp, "%d %d %d", &i, &j, &k) == 3) {
	  if (i >= Max_Pen || i < 0) {
	    fprintf(stderr, " Pen table Id >= Max_Pen\n") ;
	    exit(-1) ;
	    }
	  Pcolor[i] = j ;
	  Pwidth[i] = k ;
	  }
	fclose(fp) ;
	}
/* --------------------------------------------- Limit_File_Read ----------- */
void	Limit_Set(dsn)
char	dsn[] ;
{
	FILE	*fp ;

	if ((fp = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open limit file '%s'.\n", dsn) ;
	  exit(-1) ;
	  }
	Limit = 0 ;
	while (1) {
	  if (fgetc(fp) == 'A') {
	    fscanf(fp, "%lf %lf %lf %lf", &Xmin, &Ymin, &Xmax, &Ymax) ;
	    Limit = 1 ;
	    break ;
	    }
	  else if (fgetc(fp) == EOF) break ;
          }
	fclose(fp) ;
	if (Limit == 0) fprintf(stderr, " Limit is not set.\n") ;
	else {
	  fprintf(stderr, " Limit min = %lf %lf\n", Xmin, Ymin) ;
	  fprintf(stderr, " Limit max = %lf %lf\n", Xmax, Ymax) ;
	  }
	}
/* --------------------------------------------- Poly_Arc_Write ------------ */
void	Poly_Write(Write_Boundary, Write_Same)
int	Write_Boundary, Write_Same ;
{
	int	i, color, pattern, p, in, idon, abs(), Same() ;
	char	hed ;
	double	x, y ;
	void	Pen_Change(), Arc_Write() ;

	if (Limit == 1) {
	  in = 0 ;
	  for (i = 0; i < Poly_Arc_n; i++) {
	    if (In[abs(Arc[i])-1] == 1) {
	      in = 1 ;
	      break ;
	      }
	    }
	  if (in == 0) return ;
	  }
	p = Arc[0] ;
	if (p == 0) return ;
	color = Color[Poly_Geo[Poly_id]] ;
	pattern = Pattern[Poly_Geo[Poly_id]] ;
	if (color == 0 && pattern == 0) return ;
	printf("n\n") ;
	if (p > 0) {
	  x = Arc_X[Z(XY_Pos[p-1])] ;
	  y = Arc_Y[Z(XY_Pos[p-1])] ;
	  }
	else {
	  x = Arc_X[Z(XY_Pos[-p]-1)] ;
	  y = Arc_Y[Z(XY_Pos[-p]-1)] ;
	  }
	idon = 0 ;
	for (i = 0; i < Poly_Arc_n; i++) {
	  if (Arc[i] == 0) {
	    if (idon > 0) printf("m %lf %lf\n", x, y) ;
	    idon++ ;
	    continue ;
	    }
	  if (Write_Boundary == 1) {
	    Pen_Change(Arc[i]) ;
	    hed = 'l' ;
	    }
	  else hed = 'm' ;
	  if (Write_Same != 1 && Same(Arc_id) == 1) {
	    if (Write_Same == -1) hed = 'm' ;
	    if (Write_Same ==  0 && Arc_Fault[Arc_id-1] == 1) hed = 'm' ;
	    }
	  if (i == 0) Arc_Write(Arc[i], 0, hed) ;
	  else {
	    if (Arc[i-1] == 0) Arc_Write(Arc[i], 0, hed) ;
	    else Arc_Write(Arc[i], 1, hed) ;
	    }
	  }
	if (idon > 0) printf("m %lf %lf\n", x, y) ;
	printf("Fb %d\n", color) ;
	printf("Fp %d\n", pattern) ;
	if (pattern != 0) printf("Fc %d\n", Pattern_Color[Poly_Geo[Poly_id]]) ;
	printf("f\n") ;
	}
/* --------------------------------------------- Arc_Write ----------------- */
void	Arc_Write(id, im, hed)
int	id, im ;
char	hed ;
{
	int	i, p, is, ie, ia, abs() ;
	char	c ;

	p = abs(id) - 1 ;
	if (id > 0) {
	  is = XY_Pos[p] ;
	  ie = XY_Pos[p+1] ;
	  ia = 1 ;
	  }
	else {
	  is = XY_Pos[p+1] - 1 ;
	  ie = XY_Pos[p] - 1 ;
	  ia = -1 ;
	  }
	if (Write_IDS == 1) {
	  printf("m %lf %lf\n", Arc_X[Z(is)], Arc_Y[Z(is)]) ;
	  printf("s %d\n", id) ;
	  }
	c = 'm' ;
	for (i = is; i != ie; i = i + ia) {
	  if (im == 0 || i != is) printf("%c %lf %lf\n", c, Arc_X[Z(i)], Arc_Y[Z(i)]) ;
	  c = hed ;
	  }
	}
/* --------------------------------------------- Header read --------------- */
void	DLG_Head_Read()
{
	int	i ;

	for (i = 0; i < 10; i++) fgets(Line, 500, DLG) ;
	for (i = 0; i < 500; i++)
	  if (Line[i] == 'D' || Line[i] == 'd') Line[i] = 'e' ;
	sscanf(Line, "%lf%lf%lf%lf", &A[0], &A[1], &A[2], &A[3]) ;
	for (i = 0; i < 5; i++) fgets(Line, 500, DLG) ;
	sscanf(Line, "%s %*d %*d %d %*d %*d %d %*d %*d %d",
	  C_NAME, &Node_n, &Poly_n, &Arc_n) ;
	fprintf(stderr, "Node_n = %d Poly_n = %d Arc_n = %d\n",
		Node_n, Poly_n, Arc_n) ;
	}
/* --------------------------------------------- Node part read ------------ */
void	DLG_Node_Read()
{
	int	i ;

	while (1) {
	  fgets(Line, 500, DLG) ;
	  if (Line[0] != 'N') fprintf(stderr, " Node record error.\n") ;
	  else break ;
	  }
	if (Line[35] == ' ') Line[35] = '0' ;
	sscanf(Line, "%*c%d %lf %lf %d %d",
	  &Node_id, &Node_x, &Node_y, &i, &Node_Arc_n) ;
	for (i = 0; i < (Node_Arc_n+11)/12; i++) fgets(Line, 500, DLG) ;
	}
/* --------------------------------------------- Poly part read ------------ */
void	DLG_Poly_Read()
{
	int	i, j, attr ;

	while (1) {
	  fgets(Line, 500, DLG) ;
	  if (Line[0] != 'A') fprintf(stderr, " Area record error. %s", Line) ;
	  else break ;
	  }
	sscanf(Line, "%*c%d %lf %lf %*d %d %*d %d %*d %d",
	  &Poly_id,&Poly_x,&Poly_y,&Poly_Arc_n,&attr,&Poly_Don_n) ;
	if (((Poly_Arc_n+11)/12)*12 > Max_ID) {
	  Max_ID = ((Poly_Arc_n+11)/12)*12 ;
	  free(Arc) ;
	  /* fprintf(stderr, " Arc alloc %d %d\n", Max_ID, Poly_Arc_n) ; */
	  if ((Arc = (int *)calloc(Max_ID, sizeof(*Arc))) == NULL) {
	    fprintf(stderr, " Can't alloc memory Arc.\n") ;
	    exit(-1) ;
	    }
	  }
	for (i = 0; i < Poly_Arc_n; i=i+12) {
	  fgets(Line, 500, DLG) ;
	  sscanf(Line, "%d %d %d %d %d %d %d %d %d %d %d %d",
	  &Arc[i  ],&Arc[i+1],&Arc[i+2],&Arc[i+3],&Arc[i+ 4],&Arc[i+ 5],
	  &Arc[i+6],&Arc[i+7],&Arc[i+8],&Arc[i+9],&Arc[i+10],&Arc[i+11]) ;
	  }
	if (attr != 0) {
	  fgets(Line, 500, DLG) ;
	  sscanf(Line, "%d %d", &i, &j) ;
	  if (i >= Max_Tbl || i < 0) {
	    fprintf(stderr, " %d Poly_Geo %d >= Max_Tbl %d\n",
	      Poly_id, i, Max_Tbl) ;
	    exit(-1) ;
	    }
	  Poly_Geo[Poly_id] = i ;
	  if (attr >  6) fgets(Line, 500, DLG) ;
	  if (attr > 12) fgets(Line, 500, DLG) ;
	  if (attr > 18) fgets(Line, 500, DLG) ;
	  }
	else {
	  Poly_Geo[Poly_id] = 1 ;
	  }
	}
/* --------------------------------------------- Poly part read ------------ */
void	DLG_Arc_Read(Write_Arc, Write_Same)
int	Write_Arc, Write_Same ;
{
	int	i, j, p, l, r ;
	double	x[3], y[3], xw, yw ;
	void	Pen_Change() ;

	while (1) {
	  fgets(Line, 500, DLG) ;
	  if (Line[0] != 'L') fprintf(stderr, " Line record error. %s", Line) ;
	  else break ;
	  }
	sscanf(Line, "%*c%d %d %d %d %d %d %d",
	  &Arc_id, &Arc_S_Node, &Arc_E_Node, &l, &r, &Arc_XY_n, &Arc_Attr) ;
	p = Arc_id - 1 ;
	if (l < 0) l = 0 ;
	if (r < 0) r = 0 ;
	Arc_L_Poly[p] = l ;
	Arc_R_Poly[p] = r ;
	In[p] = 0 ;
	XY_Pos[0] = 0 ;
	XY_Pos[Arc_id] = XY_Pos[p] + Arc_XY_n ;
	if (XY_Pos[Arc_id] > Max_Point) {
	  i = Max_Point / Pint ;
	  if (i >= Max_Buff) {
	    fprintf(stderr, " Point Buffer dim overflow.\n") ;
	    exit(-1) ;
	    }
	  /* fprintf(stderr, " Arc_X/Arc_Y alloc %d\n", i) ; */
	  if ((Arc_X[i] = (double *)calloc(Pint, sizeof(*Arc_X[i]))) == NULL) {
	    fprintf(stderr, " Can't alloc memory Arc_X.\n") ;
	    exit(-1) ;
	    }
	  if ((Arc_Y[i] = (double *)calloc(Pint, sizeof(*Arc_Y[i]))) == NULL) {
	    fprintf(stderr, " Can't alloc memory Arc_Y.\n") ;
	    exit(-1) ;
	    }
	  Max_Point = Max_Point + Pint ;
	  }
	for (i = XY_Pos[p]; i < XY_Pos[Arc_id]; i++) {
	  j = (i-XY_Pos[p]) % 3 ;
	  if (j == 0) {
	    fgets(Line, 500, DLG) ;
	    sscanf(Line, "%lf%lf%lf%lf%lf%lf",
	      &x[0],&y[0],&x[1],&y[1],&x[2],&y[2]) ;
	    }
	  Arc_X[Z(i)] = A[0] * x[j] + A[1] * y[j] + A[2] ;
	  Arc_Y[Z(i)] = A[0] * y[j] - A[1] * x[j] + A[3] ;
	  if (To_Second == 1) {
	    Arc_X[Z(i)] = Arc_X[Z(i)] * 3600.0 ;
	    Arc_Y[Z(i)] = Arc_Y[Z(i)] * 3600.0 ;
	    }
	  }
	if (Arc_Attr > 0) {
	  fgets(Line, 500, DLG) ;
	  sscanf(Line, "%d", &i) ;
	  if (i < 0) i = 0 ;
	  if (i >= Max_Pen) {
	    fprintf(stderr, " Arc_Fault (%d) >= Max_Pen (%d) %d\n",
		i, Max_Pen, Arc_id) ;
	    exit(-1) ;
	    }
	  Arc_Fault[p] = i ;
	  }
	else {
	  Arc_Fault[p] = 1 ;
	  }

	if (Limit == 1) {
	  In[p] = 0 ;
	  for (i = XY_Pos[p]+1; i < XY_Pos[Arc_id]; i++) {
	    if (In[p] == 0) {
	      xw = (Arc_X[Z(i)] - Arc_X[Z(i-1)]) / 2.0 + Arc_X[Z(i-1)] ;
	      yw = (Arc_Y[Z(i)] - Arc_Y[Z(i-1)]) / 2.0 + Arc_Y[Z(i-1)] ;
	      if (xw > Xmin && yw > Ymin && xw < Xmax && yw < Ymax) {
	        In[p] = 1 ;
	        break ;
	        }
	      }
	    }
	  if (In[p] == 0) return ;
	  }
	if (Write_Arc == 1 && Pwidth[Arc_Fault[p]] != 0) {
	  if (Write_Same != 1 && Same(Arc_id) == 1) {
	    if (Write_Same == -1) return ;
	    if (Write_Same ==  0 && Arc_Fault[p] == 1) return ;
	    }
	  Pen_Change(Arc_id) ;
	  Arc_Write(Arc_id, 0, 'l') ;
	  }
	}
/* --------------------------------------------- Same ? -------------------- */
int	Same(id)
int	id ;
{
	int	l, r, p, abs() ;

	p = abs(id) - 1 ;
	l = Poly_Geo[Arc_L_Poly[p]] ;
	r = Poly_Geo[Arc_R_Poly[p]] ;
	if (Coast == 1 && (l == 0  || r == 0))  return(0) ;
	if (Color[l] == Color[r] && Pattern[l] == Pattern[r]) return(1) ;
	return(0) ;
	}
/* --------------------------------------------- Pen_Change ---------------- */
void	Pen_Change(id)
int	id ;
{
	int	n, abs() ;

	n = Arc_Fault[abs(id)-1] ;
	if (Pwidth[n] == -1) return ;
	if (Pcolor[n] != Color_old) {
	  printf("Lc %d\n", Pcolor[n]) ;
	  Color_old = Pcolor[n] ;
	  }
	if (Pwidth[n] != Width_old) {
	  printf("Lw %d\n", Pwidth[n]) ;
	  Width_old = Pwidth[n] ;
	  }
	}
