/*
*  Program name:             gauss  ver. 2.1
*
*  Program function:         GAUSS map projection, forward routine
*                            (lat, lon) ==>> (x, y)
*                            Spheroid = Bessel
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            September 10, 1990
*
*  Date of revision:         March 24, 1992 by Y.Murata
*                            June 4, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*                            December 17, 2003 add GRS option by Y. Murata
*
*  Arguments:                1. 'KEI' number 1 - 17
*
*  Input:                    JKG data in second (from stdin)
*                                 project only 'l' 'm' 'A' 'W' data
*
*  Output:                   projected JKG data in meter (to stdout)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

double	lon0, lat0, Y0 ;
double	rm0, pi, pis, daa, dee, deee, da, db, dc, dd, de, df ;
int	grs ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	lon, lat, lon2, lat2, x, y ;
	char	hed, txt[500] ;
	int	kei, atoi() ;
	void	GAUSS_INIT(), GAUSS() ;

	grs = 0 ;
	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr, " USAGE: gauss {KEI-number}\n") ;
		return(0) ;
		}
	if (argc >= 2) kei = atoi(argv[1]) ; else kei = 1 ;
	     if (kei ==  1) { lat0 = 33.0 ; lon0 = 129.50 ; }
	else if (kei ==  2) { lat0 = 33.0 ; lon0 = 131.00 ; }
	else if (kei ==  3) { lat0 = 36.0 ; lon0 = 132.00 + 1.0 / 6.0 ; }
	else if (kei ==  4) { lat0 = 33.0 ; lon0 = 133.50 ; }
	else if (kei ==  5) { lat0 = 36.0 ; lon0 = 134.00 + 2.0 / 6.0 ; }
	else if (kei ==  6) { lat0 = 36.0 ; lon0 = 136.00 ; }
	else if (kei ==  7) { lat0 = 36.0 ; lon0 = 137.00 + 1.0 / 6.0 ; }
	else if (kei ==  8) { lat0 = 36.0 ; lon0 = 138.50 ; }
	else if (kei ==  9) { lat0 = 36.0 ; lon0 = 139.00 + 5.0 / 6.0 ; }
	else if (kei == 10) { lat0 = 40.0 ; lon0 = 140.00 + 5.0 / 6.0 ; }
	else if (kei == 11) { lat0 = 44.0 ; lon0 = 140.25 ; }
	else if (kei == 12) { lat0 = 44.0 ; lon0 = 142.25 ; }
	else if (kei == 13) { lat0 = 44.0 ; lon0 = 144.25 ; }
	else if (kei == 14) { lat0 = 26.0 ; lon0 = 142.00 ; }
	else if (kei == 15) { lat0 = 26.0 ; lon0 = 127.50 ; }
	else if (kei == 16) { lat0 = 26.0 ; lon0 = 124.00 ; }
	else if (kei == 17) { lat0 = 26.0 ; lon0 = 131.00 ; }
	else {
		fprintf(stderr, " Kei number (1-17) error '%d'.\n", kei) ;
		return(-1) ;
		}
	if (argc >= 3 && strcmp("GRS80", argv[2])==0) grs = 1 ;
	if (argc >= 3 && strcmp("grs80", argv[2])==0) grs = 1 ;
	if (argc >= 3 && strcmp("grs", argv[2])==0) grs = 1 ;

	GAUSS_INIT() ;

	while ((fgets(txt, 500, stdin)) != NULL) {
		if (txt[0] == 'l' || txt[0] == 'm') {
			if (sscanf(txt, "%c %lf %lf",
				&hed, &lon, &lat) != 3) {
				fprintf(stderr, " JKG l/m data error.\n") ;
				break ;
				}
			GAUSS(&lon, &lat, &x, &y) ;
			printf("%c %9.2lf %10.2lf\n", hed, x, y) ;
			}
		else if(txt[0] == 'A' || txt[0] == 'W') {
			if (sscanf(txt, "%c %lf %lf %lf %lf",
				&hed, &lon, &lat, &lon2, &lat2) != 5) {
				fprintf(stderr, " JKG A/W data error.\n") ;
				break ;
				}
			GAUSS(&lon, &lat, &x, &y) ;
			printf("%c %9.2lf %10.2lf", hed, x, y) ;
			GAUSS(&lon2, &lat2, &x, &y) ;
			printf(" %9.2lf %10.2lf\n", x, y) ;
			}
		else printf("%s", txt) ;
		}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	GAUSS_INIT()
{
	double	lon, lat, x, y ;
	double	f, e2, e4, e6, e8, e10, e12, e14, e16 ;
	void	GAUSS() ;

	rm0   = 0.9999 ;
	pi    = 1.745329251993889e-2 ;
	pis   = pi / 3600.0 ;
	if (grs == 0) {
		daa = 6377397.155 ;
		f = 1.0 / 299.1528128 ;
		}
	else {
		fprintf(stderr, "Set to GRS80\n") ;
		daa = 6378137.000 ;
		f = 1.0 / 298.257222101 ;
		}
	dee = f * (2.0 - f) ;
	deee = dee / (1.0 - dee) ;
	e2  = dee ;
	e4  = e2  * e2 ;
	e6  = e4  * e2 ;
	e8  = e6  * e2 ;
	e10 = e8  * e2 ;
	e12 = e10 * e2 ;
	e14 = e12 * e2 ;
	e16 = e14 * e2 ;
	da = 1.0
	  + 3.0 / 4.0 * e2
	  + 45.0 / 64.0 * e4
	  + 175.0 / 256.0 * e6
	  + 11025.0 / 16384.0 * e8
	  + 43659.0 / 65536.0 * e10
	  + 693693.0 / 1048576.0 * e12
	  + 19324305.0 / 29360128.0 * e14
	  + 4927697775.0 / 7516192768.0 * e16 ;
	db =
	  + 3.0 / 4.0 * e2
	  + 15.0 / 16.0 * e4
	  + 525.0 / 512.0 * e6
	  + 2205.0 / 2048.0 * e8
	  + 72765.0 / 65536.0 * e10
	  + 297297.0 / 262144.0 * e12
	  + 135270135.0 / 117440512.0 * e14
	  + 547521975.0 / 469762048.0 * e16 ;
	dc =
	  + 15.0 / 64.0 * e4
	  + 105.0 / 256.0 * e6
	  + 2205.0 / 4096.0 * e8
	  + 10395.0 / 16384.0 * e10
	  + 1486485.0 / 2097152.0 * e12
	  + 45090045.0 / 58720256.0 * e14
	  + 766530765.0 / 939524096.0 * e16 ;
	dd =
	  + 35.0 / 512.0 * e6
	  + 315.0 / 2048.0 * e8
	  + 31185.0 / 131072.0 * e10
	  + 165165.0 / 524288.0 * e12
	  + 45090045.0 / 117440512.0 * e14
	  + 209053845.0 / 469762048.0 * e16 ;
	de =
	  + 315.0 / 16384.0 * e8
	  + 3465.0 / 65536.0 * e10
	  + 99099.0 / 1048576.0 * e12
	  + 4099095.0 / 29360128.0 * e14
	  + 348423075.0 / 1879048192.0 * e16 ;
	df =
	  + 693.0 / 131072.0 * e10
	  + 9009.0 / 524288.0 * e12
	  + 4099095.0 / 117440512.0 * e14
	  + 26801775.0 / 469762048.0 * e16 ;
	lon   = lon0 * 3600.0 ;
	lat   = lat0 * 3600.0 ;
	Y0    = 0.0 ;
	lon0  = lon0 * pi ;
	GAUSS(&lon, &lat, &x, &y) ;
	Y0 = y ;
	}
/* -------------------------------------------------------------------------- */
void	GAUSS(dlon, dlat, x, y)
double	*dlon, *dlat, *x, *y ;
{
	double	lon, lat, lon_2, lat_c, lat_s, lat_c_2, lat_s_2 ;
	double	dt, dt2, dt4, dt6, das ;
	double	dbs, dcs, dds, des, dfs, ds, dn, deta ;
	double	tmp0, tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7, tmp8 ;

	lon     = *dlon * pis - lon0 ;
	lat     = *dlat * pis ;
	lon_2   = lon * lon ;
	lat_c   = cos(lat) ;
	lat_s   = sin(lat) ;
	lat_c_2 = lat_c * lat_c ;
	lat_s_2 = lat_s * lat_s ;

	dt = tan(lat) ;
	dt2 = dt * dt ;
	dt4 = dt2 * dt2 ;
	dt6 = dt4 * dt2 ;
	das = da * lat ;

	dbs = db / 2.0  * sin( 2.0 * lat) ;
	dcs = dc / 4.0  * sin( 4.0 * lat) ;
	dds = dd / 6.0  * sin( 6.0 * lat) ;
	des = de / 8.0  * sin( 8.0 * lat) ;
	dfs = df / 10.0 * sin(10.0 * lat) ;

	ds = daa * (1.0 - dee) * (das - dbs + dcs - dds + des - dfs) ;

	dn = daa/ sqrt(1.0 - dee * lat_s_2) ;
	deta = deee * lat_c_2 ;

	tmp0 = lon_2 * lat_c_2 ;
	tmp1 = lon_2 * dn * lat_s * lat_c ;
	tmp2 = 5.0 - dt2 + 9.0 * deta + 4.0 * deta * deta ;
	tmp3 = 61.0 - 58.0 * dt2 + dt4 + 270.0 * deta
			- 330.0 * dt2 * deta ;
	tmp4 = -1385.0 + 3111.0 * dt2 - 543.0 * dt4 + dt6 ;
	tmp5 = lon * dn * lat_c ;
	tmp6 = 1.0 - dt2 + deta ;
	tmp7 = 5.0 - 18.0 * dt2 + dt4 + 14.0 * deta -
			58.0 * dt2 * deta ;
	tmp8 = -61.0 + 479.0 * dt2 - 179.0 * dt4 + dt6 ;

	*x = tmp5 * (1.0 + tmp0 * (tmp6 / 6.0 + tmp0 * (
	  tmp7 / 120.0 - tmp0 * (tmp8 / 5040.0)))) ;
	*x = *x * rm0 ;
	*y = ds + tmp1 * (0.5 + tmp0 * (tmp2 / 24.0 +
	  tmp0 * (tmp3 / 720.0 - tmp0 * (tmp4 / 40320.0)))) ;
	*y = *y * rm0 - Y0 ;
	}
