/*
*  Program name:             jkg  ver. 2.0
*
*  Program function:         JKG reading routine.
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         May 27, 1995 rewrite using C by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*                            May 20, 2004 Change Fb, Fc option by Y. Murata
*
*  Arguments:                -s Scale (default : auto)
*                            -r plot angle (default : 0)
*                            -c color table file name (JK-CMY)
*                            -p pattern table file (JK-PATTERN)
*
*  Input:                    JKG data ( from standard input )
*                                    see file "jkg.smp" for example.
*
*  Output:                   depends on the output device
*/

#include <stdio.h>
#include <string.h>
#include "jk_macro.h"

long	Ndev, Center ;
double	Xmin, Ymin, Xmax, Ymax, Xwid, Ywid ;
double	Scale, Xstr, Ystr, Rad ;
double	Sinr, Cosr, Sinrs, Cosrs, Xscon, Yscon ;

long	Npaint ;
float	Xpaint[MAXD], Ypaint[MAXD] ;

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	float	x, y, x1, y1, x2, y2 ;
	long	ipaint, i ;
	long	cpaint[MAXD], iw ;
	double	dx1, dy1, dx2, dy2, dx3, dy3, sin(), cos(), asin(), atof() ;
	double	xmin2, ymin2, xmax2, ymax2 ;
	char	head, c[500] ;
	void	conv1(), conv2(), Scale_Set() ;

	long	mpat, mcol ;			/* For marks */
	float	msiz ;
	long	scol ;				/* For strings */
	float	ssiz, srad, shor, sver ;
	long	lwid, lcol, lpat ;		/* For lines */
	float	llen ;
	long	fpat, fbkc, fptc ;		/* For areas */

						/* device depended routine */
	void	plotst_(), chgpen_(), drwstr_(), drwmrk_(),
		Area_Fill(), plot_(), window_(), ploten_(),
		coltbl_(), pattbl_() ;
/* -------------------------------------------------------------------------- */
	plotst_(&Xstr, &Ystr, &Xwid, &Ywid, &Ndev, argc, argv) ;
	fprintf(stderr, " Paper size = %lf %lf\n", Xwid, Ywid) ;

	ipaint = 0 ;
	Npaint = 0 ;
	Rad  = 0.0 ;
	Sinr = 0.0 ;
	Cosr = 1.0 ;
	Xmin = 0.0 ;
	Xmax = 0.0 ;
	Ymin = 0.0 ;
	Ymax = 0.0 ;

	fbkc = 2 ;
	fptc = 0 ;
	fpat = 0 ;
	mcol = 1 ;
	msiz = 10.0 ;
	mpat = 1 ;
	lcol = 0 ;
	lwid = 1 ;
	lpat = 0xFF ;
	llen = 1.0 ;
	scol = 1 ;
	ssiz = 10.0 ;
	srad = 0.0 ;
	shor = 0.0 ;
	sver = 0.0 ;
/* -------------------------------------------------------------------------- */
	Center = 0 ;
	for ( i = 1; i < argc; i++ ) {
	       if (!strcmp(argv[i], "-s"))   Scale = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-r"))   Rad   = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-cen")) Center = 1 ;
	  else if (!strcmp(argv[i], "-c"))   coltbl_(argv[++i]) ;
	  else if (!strcmp(argv[i], "-p"))   pattbl_(argv[++i]) ;
	  }
	chgpen_(&lwid, &lcol, &lpat, &llen) ;
/* -------------------------------------------------------------------------- */
	while ( (fgets(c, 500, stdin)) != NULL ) {
	  head = c[0] ;
	  c[0] = ' ' ;
	  if (c[strlen(c)-1] == '\n') c[strlen(c)-1] = '\0' ;
/* -------------------------------------------------------------------------- */
	  if (head == 'l' || head == 'm') {
	    if (head == 'l') iw = 2 ;
	    else iw = 3 ;
	    sscanf(c, "%lf %lf", &dx1, &dy1) ;
	    conv1(dx1, dy1, &x, &y) ;
	    if (ipaint == 0) plot_(&x, &y, &iw) ;
	    else {
	      Xpaint[Npaint] = x ;
	      Ypaint[Npaint] = y ;
	      cpaint[Npaint] = iw ;
	      Npaint++ ;
	      if (Npaint > MAXD-1) {
	        fprintf(stderr,	" Number of vertex > MAXD.\n") ;
	        return(-1) ;
	        }
	      }
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'n') {
	    ipaint = 1 ;
	    Npaint = 0 ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'f') {
	    Area_Fill(Xpaint, Ypaint, &Npaint, &fbkc, &fpat, &fptc) ;
	    cpaint[Npaint+1] = 3 ;
	    for (i = 0; i < Npaint; i++)
	      if (cpaint[i] == 2 || cpaint[i+1] == 2)
	        plot_(&Xpaint[i], &Ypaint[i], &cpaint[i]) ;
	    ipaint = 0 ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'F') {
	         if (c[1] == 'p') sscanf(c, "%*c%*c %ld", &fpat) ;
	    else if (c[1] == 'b') sscanf(c, "%*c%*c %ld", &fbkc) ;
	    else if (c[1] == 'c') sscanf(c, "%*c%*c %ld", &fptc) ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 's') {
	    iw = strlen(c) - 2 ;
	    drwstr_(&x, &y, &ssiz, &srad, &shor, &sver, &scol, &iw, &c[2]) ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'p') {
	    drwmrk_(&x, &y, &mpat, &msiz, &mcol) ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'L') {
	         if (c[1] == 'w') sscanf(c, "%*c%*c %ld", &lwid) ;
	    else if (c[1] == 'c') sscanf(c, "%*c%*c %ld", &lcol) ;
	    else if (c[1] == 'p') sscanf(c, "%*c%*c %ld", &lpat) ;
	    else if (c[1] == 'l') sscanf(c, "%*c%*c %f",  &llen) ;
	    chgpen_(&lwid, &lcol, &lpat, &llen) ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'r') {
	    if (c[1] == 'l') iw = 2 ;
	    else iw = 3 ;
	    c[1] = ' ' ;
	    sscanf(c, "%lf %lf", &dx2, &dy2) ;
	    conv2(dx1, dy1, dx2, dy2, &x, &y) ;
	    if (ipaint == 0) plot_(&x, &y, &iw) ;
	    else {
	      Xpaint[Npaint] = x ;
	      Ypaint[Npaint] = y ;
	      cpaint[Npaint] = iw ;
	      Npaint++ ;
	      if (Npaint >= MAXD) {
	        fprintf(stderr,	" Number of vertex >= MAXD.\n") ;
	        return(-1) ;
	        }
	      }
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'S') {
	         if (c[1] == 's') sscanf(c, "%*c%*c %f",  &ssiz) ;
	    else if (c[1] == 'h') sscanf(c, "%*c%*c %f",  &shor) ;
	    else if (c[1] == 'v') sscanf(c, "%*c%*c %f",  &sver) ;
	    else if (c[1] == 'c') sscanf(c, "%*c%*c %ld", &scol) ;
	    else if (c[1] == 'r') {
	      sscanf(c, "%*c%*c %f",  &srad) ;
	      srad = srad + Rad ;
	      }
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'M') {
	         if (c[1] == 'p') sscanf(c, "%*c%*c %ld", &mpat) ;
	    else if (c[1] == 's') sscanf(c, "%*c%*c %f",  &msiz) ;
	    else if (c[1] == 'c') sscanf(c, "%*c%*c %ld", &mcol) ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'A') {
	    if (Xmin != Xmax) fprintf(stderr, "Record A is already set!\n") ;
	    else {
	      sscanf(c, "%lf %lf %lf %lf", &Xmin, &Ymin, &Xmax, &Ymax) ;
	      if (Rad == 0.0) Scale_Set() ;
	      else {
	        Sinr = sin(Rad / 90.0 * asin(1.0)) ;
	        Cosr = cos(Rad / 90.0 * asin(1.0)) ;
	        xmin2 =  9.9e29 ;
	        ymin2 =  9.9e29 ;
	        xmax2 = -9.9e29 ;
	        ymax2 = -9.9e29 ;

/*	        for(ix = 0; ix < 2; ix++) for(iy = 0; iy < 2; iy++) {	*/
/*	          dx1 = Xmin + (double)ix * (Xmax - Xmin) ;	*/
/*	          dy1 = Ymin + (double)iy * (Ymax - Ymin) ;	*/
/*	          dx2 = dx1 * Cosr - dy1 * Sinr ;	*/
/*	          dy2 = dx1 * Sinr + dy1 * Cosr ;	*/
/*	          xmin2 = min(xmin2, dx2) ;	*/
/*	          ymin2 = min(ymin2, dy2) ;	*/
/*	          xmax2 = max(xmax2, dx2) ;	*/
/*	          ymax2 = max(ymax2, dy2) ;	*/
/*	          }	*/
	        dx2 = Xmin * Cosr - Ymin * Sinr ;
	        dy2 = Xmin * Sinr + Ymin * Cosr ;
	        xmin2 = min(xmin2, dx2) ;
	        ymin2 = min(ymin2, dy2) ;
	        xmax2 = max(xmax2, dx2) ;
	        ymax2 = max(ymax2, dy2) ;
	        dx2 = Xmax * Cosr - Ymax * Sinr ;
	        dy2 = Xmax * Sinr + Ymax * Cosr ;
	        xmin2 = min(xmin2, dx2) ;
	        ymin2 = min(ymin2, dy2) ;
	        xmax2 = max(xmax2, dx2) ;
	        ymax2 = max(ymax2, dy2) ;

	        Xmin = xmin2 ;
	        Ymin = ymin2 ;
	        Xmax = xmax2 ;
	        Ymax = ymax2 ;
	        Scale_Set() ;
	        }
	      }
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'W') {
	    sscanf(c, "%lf %lf %lf %lf", &dx2, &dy2, &dx3, &dy3) ;
	    conv1(dx2, dy2, &x1, &y1) ;
	    conv1(dx3, dy3, &x2, &y2) ;
	    dx2 = min(x1, x2) ;
		dx3 = max(x1, x2) ;
	    dy2 = min(y1, y2) ;
		dy3 = max(y1, y2) ;
	    x1 = dx2 ;
		y1 = dy2 ;
		x2 = dx3 ;
		y2 = dy3 ;
	    iw = 0 ;
	    window_(&iw, &x1, &y1, &x2, &y2) ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == 'R') {
	    iw = 1 ; x1 = 0.0 ; y1 = 0.0 ; x2 = 0.0 ; y2 = 0.0 ;
	    window_(&iw, &x1, &y1, &x2, &y2) ;
	    }
/* -------------------------------------------------------------------------- */
	  else if (head == '%') {
	    printf("%s\n", &c[1]) ;
	    }
/* -------------------------------------------------------------------------- */
	  }
	ploten_() ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
void	conv1(dxi, dyi, dxo, dyo)
double	dxi, dyi ;
float	*dxo, *dyo ;
{
	*dxo = dxi * Cosrs - dyi * Sinrs + Xscon ;
	*dyo = dxi * Sinrs + dyi * Cosrs + Yscon ;
	}
/* -------------------------------------------------------------------------- */
void	conv1_r(dxi, dyi, dxo, dyo)
double	dxi, dyi, *dxo, *dyo ;
{
	double	sinrs , cosrs, sin(), cos(), asin() ;

	sinrs = sin(-Rad / 90.0 * asin(1.0)) / Scale ;
	cosrs = cos(-Rad / 90.0 * asin(1.0)) / Scale ;
	*dxo = (dxi - Xscon) * cosrs - (dyi-Yscon) * sinrs ;
	*dyo = (dxi - Xscon) * sinrs + (dyi-Yscon) * cosrs ;
	}
/* -------------------------------------------------------------------------- */
void	conv2(dxi, dyi, dxs, dys, dxo, dyo)
double	dxi, dyi, dxs, dys ;
float	*dxo, *dyo ;
{
	*dxo = (dxi * Scale + dxs) * Cosr - (dyi * Scale + dys) * Sinr ;
	*dyo = (dxi * Scale + dxs) * Sinr + (dyi * Scale + dys) * Cosr ;
	*dxo = *dxo + Xscon ;
	*dyo = *dyo + Yscon ;
	}
/* -------------------------------------------------------------------------- */
void	Scale_Set()
{
	if (Scale == 0.0) {
	  Scale = min(Xwid/(Xmax-Xmin), Ywid/(Ymax-Ymin)) ;
	  fprintf(stderr, " Scale = 1 / %lf\n", 100.0/Scale) ;
	  }
	else {
	  Scale = 1.0 / Scale * 100.0 ;
	  fprintf(stderr, " Plot size = %lf %lf\n",
	    (Xmax-Xmin)*Scale, (Ymax-Ymin)*Scale) ;
	  if ((Xmax-Xmin)*Scale > Xwid)
	    fprintf(stderr, " ### Plot x-size (%lf) > Paper x-size(%lf)\n",
	    (Xmax-Xmin)*Scale, Xwid) ;
	  if ((Ymax-Ymin)*Scale > Ywid)
	    fprintf(stderr, " ### Plot y-size (%lf) > Paper y-size(%lf)\n",
	    (Ymax-Ymin)*Scale, Ywid) ;
	  }
	if (Center == 1) {
	  Xstr = (Xwid - (Xmax-Xmin)*Scale) / 2.0 ;
	  Ystr = (Ywid - (Ymax-Ymin)*Scale) / 2.0 ;
	  fprintf(stderr, " Move to center %lf %lf\n", Xstr, Ystr) ;
	  }
	Sinrs = Sinr * Scale ;
	Cosrs = Cosr * Scale ;
	Xscon = -Xmin * Scale + Xstr ;
	Yscon = -Ymin * Scale + Ystr ;
	}
