/*
*  Program name:             latlon  ver. 2.0
*
*  Program function:         Latitude longitude line generate
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            December 26, 1988
*
*  Date of last revision:    May 28, 1991 by Y. Murata
*                            Feb 2, 1993 Change Arguments by Y. Murata
*                            June 5, 1995 for JKG by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. lower-left  corner longitude (unit:deg.min)
*                            2. lower-left  corner latitude  (unit:deg.min)
*                            3. upper-right corner longitude (unit:deg.min)
*                            4. upper-right corner latitude  (unit:deg.min)
*                            5. interval of longitude (unit:second)
*                            6. interval of latitude  (unit:second)
*                            -llon segment-legnth (default 1/10 interval)
*                            -llat segment-legnth (default 1/10 interval)
*                            -tlon thick line interval (default 0)
*                            -tlat thick line interval (default 0)
*                            -m character size (default 0)
*                            -s character size (default 0)
*                            -f draw frame (default none)
*
*  Input:                    none
*
*  Output:                   JKG longitude and latitude in second to stdout
*/

#include <stdio.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	minlat, maxlat, minlon, maxlon, atof(), deg, min, mojisz, w ;
	long	latmin, latmax, lonmin, lonmax ;
	long	intlat, intlon, lenlat, lenlon, thklat, thklon ;
	long	i, j, k, start, frame ;
	int	atoi() ;

	if (argc < 2 || strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr, " USAGE: latlon {minlon minlat maxlon maxlat ") ;
	  fprintf(stderr, "intlon intlat}\n\t") ;
	  fprintf(stderr, "{-llon length -llat length -tlon interval ") ;
	  fprintf(stderr, "-tlat interval -m char-size}\n") ;
	  return(0) ;
	  }
	if (argc > 1) minlon = atof(argv[1]) ; else minlon  =    0.0 ;
	if (argc > 2) minlat = atof(argv[2]) ; else minlat  =  -90.0 ;
	if (argc > 3) maxlon = atof(argv[3]) ; else maxlon  =  360.0 ;
	if (argc > 4) maxlat = atof(argv[4]) ; else maxlat  =   90.0 ;
	if (argc > 5) intlon = atoi(argv[5]) ; else intlon  = 3600L  ;
	if (argc > 6) intlat = atoi(argv[6]) ; else intlat  = 3600L  ;
	lenlon = intlon / 10L ;
	lenlat = intlat / 10L ;
	thklon = 0L ;
	thklat = 0L ;
	frame = 0L ;
	start = 0L ;
	mojisz = 0.0 ;
	for (i = 7; i < argc; i++) {
	       if (!strcmp(argv[i], "-llon")) lenlon = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-llat")) lenlat = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-tlon")) thklon = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-tlat")) thklat = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-m")   ) mojisz = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-f")   ) frame = 1L ;
	  else if (!strcmp(argv[i], "-s")   ) start = 1L ;
	  }
	if (lenlon <= 0 || lenlat <= 0 ||
/*	    thklon%intlon != 0 || thklat%intlat != 0 ||
	    intlon%lenlon != 0 || intlat%lenlat != 0 || */
	    minlon >= maxlon || minlat >= maxlat ) {
	  fprintf(stderr, " Argument parameters error.\n") ;
	  fprintf(stderr, " minlon, minlat = %8.2f %8.2f\n", minlon, minlat) ;
	  fprintf(stderr, " maxlon, maxlat = %8.2f %8.2f\n", maxlon, maxlat) ;
	  fprintf(stderr, " intlon, intlat = %8ld %8ld\n", intlon, intlat) ;
	  fprintf(stderr, " lenlon, lenlat = %8ld %8ld\n", lenlon, lenlat) ;
	  fprintf(stderr, " thklon, thklat = %8ld %8ld\n", thklon, thklat) ;
	  return(-1) ;
	  }

	deg = (long)(minlat) ;
	min = (minlat - deg) * 100.0 ;
	if (minlat > 0) latmin = deg * 3600.0 + min * 60.0 + 0.5 ;
	else            latmin = deg * 3600.0 + min * 60.0 - 0.5 ;
	deg = (long)(maxlat) ;
	min = (maxlat - deg) * 100.0 ;
	if (maxlat > 0) latmax = deg * 3600.0 + min * 60.0 + 0.5 ;
	else            latmax = deg * 3600.0 + min * 60.0 - 0.5 ;
	deg = (long)(minlon) ;
	min = (minlon - deg) * 100.0 ;
	if (minlon > 0) lonmin = deg * 3600.0 + min * 60.0 + 0.5 ;
	else            lonmin = deg * 3600.0 + min * 60.0 - 0.5 ;
	deg = (long)(maxlon) ;
	min = (maxlon - deg) * 100.0 ;
	if (maxlon > 0) lonmax = deg * 3600.0 + min * 60.0 + 0.5 ;
	else            lonmax = deg * 3600.0 + min * 60.0 - 0.5 ;
	if (lonmax <= lonmin) lonmax = lonmax + 360L * 3600L ;

	if (intlat != 0) {
	  if (mojisz != 0.0) {
	    printf("Ss %lf\n", mojisz) ;
	    printf("Sh 1.1\n") ;
	    printf("Sv 0.5\n") ;
	    printf("Sr 0.0\n") ;
	    }
	  for (i = latmin; i <= latmax; i++) {
	    if (i == latmin) goto P1 ;
	    if (i == latmax) goto P1 ;
	    if (start == 0 && i%intlat == 0) goto P1 ;
	    if (start == 1 && (i-latmin)%intlat == 0) goto P1 ;
	    continue ;
P1:	    if (thklat > 0) {
	      if (i%thklat == 0) {
	        printf("Lw 2\n") ;
	        if (mojisz != 0.0) {
	          printf("m %ld %ld\n", lonmin, i) ;
	          if (i%3600L == 0) printf("s %ld N\n", i/3600L) ;
	          else {
	            w = (double)(i/3600) ;
	            w = w + (double)(i%3600) / 6000.0 ;
	            printf("s %5.2lf N\n", w) ;
	            }
	          }
	        }
	      else printf("Lw 1\n") ;
	      }
	    printf("m %ld %ld\n", lonmin, i) ;
	    for (j = lonmin+lenlon; j <= lonmax; j=j+lenlon) {
	      k = j % 1296000L ;
	      if (j == 1296000L) k = 1295999L ;
	      printf("l %ld %ld\n", k, i) ;
	      if (j < lonmax && j+lenlon > lonmax)
	      j = lonmax - lenlon ;
	      }
	    if (frame==1L && i+intlat>latmax && i!=latmax) i = latmax - intlat ;
	    }
	  }

	if (intlon != 0) {
	  if (mojisz != 0.0) {
	    printf("Ss %lf\n", mojisz) ;
	    printf("Sh 0.5\n") ;
	    printf("Sv 1.4\n") ;
	    printf("Sr 0.0\n") ;
	    }
	  for (j = lonmin; j <= lonmax; j++) {
	    if (j == lonmin) goto P2 ;
	    if (j == lonmax) goto P2 ;
	    if (start == 0 && j%intlon == 0) goto P2 ;
	    if (start == 1 && (j-lonmin)%intlon == 0) goto P2 ;
	    continue ;
P2:	    if (thklon > 0) {
	      if (j%thklon == 0) {
	        printf("Lw 2\n") ;
	        if (mojisz != 0.0) {
	          printf("m %ld %ld\n", j, latmin) ;
	          if (j%3600L == 0) printf("s %ld E\n", j/3600L) ;
	          else {
	            w = (double)(j/3600) ;
	            w = w + (double)(j%3600) / 6000.0 ;
	            printf("s %6.2lf E\n", w) ;
	            }
	          }
	        printf("Lw 1\n") ;
	        }
	      }
	    printf("m %ld %ld\n", j, latmin) ;
	    for (i = latmin+lenlat; i <= latmax; i=i+lenlat) {
	      k = j % 1296000L ;
	      if (j == 1296000L) k = 1295999L ;
	      printf("l %ld %ld\n", k, i) ;
	      if (i < latmax && i+lenlat > latmax) i = latmax - lenlat ;
	      }
	    if (frame==1L && j+intlon>lonmax && j!=lonmax) j = lonmax - intlon ;
	    }
	  }
	return(0) ;
	}
