/*
*  Program name:             meshline  ver. 2.1
*
*  Program function:         mesh line data generate
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         June 30, 1995 rewrite using C by Y. Murata
*                            October 3, 1997 add character size option
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1: mesh lower left x coordinate
*                            2: mesh lower left y coordinate
*                            3: mesh width
*                            4: mesh height
*                            5: mesh number of x
*                            6: mesh number of y
*                            7-: -head
*                            7-: -m character size
*
*  Input:                    none
*
*  Output:                   JKG data (to stdout)
*/

#include <stdio.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	xmin, ymin, xmax, ymax, width, height, atof(), x, y, size=0.0 ;
	long	i, nx, ny, ix, iy ;
	int	atoi() ;
	char	str[100] ;

	if (argc < 7) {
	  fprintf(stderr,
	    " USAGE: meshline xmin ymin width height nx ny [-head -m size]\n") ;
	  return(0) ;
	  }
	xmin   = atof(argv[1]) ;
	ymin   = atof(argv[2]) ;
	width  = atof(argv[3]) ;
	height = atof(argv[4]) ;
	nx     = atoi(argv[5]) ;
	ny     = atoi(argv[6]) ;
	if (width == 0.0 || height == 0.0 || nx < 1 || ny < 1) {
	  fprintf(stderr, " Argument error.\n") ;
	  fprintf(stderr, " USAGE: meshline xmin ymin width height nx ny\n") ;
	  return(-1) ;
	  }

	xmax = xmin + (double)nx * width ;
	ymax = ymin + (double)ny * height ;
	if (argc > 7) {
	  for (i = 7; i < argc; i++) {
	    if (!strcmp(argv[i], "-head"))
	      printf("A %lf %lf %lf %lf\n", xmin, ymin, xmax, ymax) ;
	    if (!strcmp(argv[i], "-m"))
	      size = atof(argv[++i]) ;
	    }
	  }

	if (size != 0.0) {
	  printf("Ss %f\n", size) ;
	  printf("Sh 0.5\n") ;
	  printf("Sv 1.3\n") ;
	  printf("Sr 0.0\n") ;
	  }
	for (ix = 0; ix <= nx; ix++) {
	  x = xmin + width * (double)ix ;
	  printf("m %lf %lf\n", x, ymin) ;
	  printf("l %lf %lf\n", x, ymax) ;
	  if (size != 0.0) {
	    printf("m %lf %lf\n", x, ymin) ;
	    sprintf(str, "s %lf\0", x) ;
	    for (i = strlen(str)-1; i > 0; i--) {
	      if (str[i] == '0') str[i] = '\0' ;
	      else break ;
	      }
	    printf("%s\n", str) ;
	    }
	  }

	if (size != 0.0) {
	  printf("Ss %f\n", size) ;
	  printf("Sh 1.3\n") ;
	  printf("Sv 0.5\n") ;
	  }
	for (iy = 0; iy <= ny; iy++) {
	  y = ymin + height * (double)iy ;
	  printf("m %lf %lf\n", xmin, y) ;
	  printf("l %lf %lf\n", xmax, y) ;
	  if (size != 0.0) {
	    printf("m %lf %lf\n", xmin, y) ;
	    sprintf(str, "s %lf\0", y) ;
	    for (i = strlen(str)-1; i > 0; i--) {
	      if (str[i] == '0') str[i] = '\0' ;
	      else break ;
	      }
	    printf("%s\n", str) ;
	    }
	  }
	return(0) ;
	}
