/*
*  Program name:             point  ver. 2.0
*
*  Program function:         point marking program
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         June 29, 1995 rewrite using C by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -x x coordinate position (default: 2 )
*                            -y y coordinate position (default: 3 )
*                            -u Item position of upper side (default: no plot)
*                            -d Item position of lower side (default: no plot)
*                            -l Item position of left side (default: no plot)
*                            -r Item position of rigth side (default: no plot)
*                            -p pattern number position
*                                   (default: fix to 1   negative: pat number )
*                            -m character size position
*                                   (default: fix to 0.1 negative: char size  )
*                            -w xmin, ymin, xmax, ymax
*
*  Input :                   JKP file (from stdin)
*
*  Output:                   JKG data (to stdout)
*/

#include <stdio.h>
#include <string.h>

#define	MAXL	20

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, j, n, nx=0, ny=0, nu=0, nd=0, nl=0, nr=0, atoi() ;
	int	mp, np=0, nm=0, w=0 ;
	float	sm=0.0 ;
	double	xmin, ymin, xmax, ymax, rx, ry, atof() ;
	char	txt[500] ;
	char	x[MAXL], y[MAXL], u[MAXL], d[MAXL], l[MAXL], r[MAXL],
		p[MAXL], m[MAXL] ;

	for (i = 1; i < argc; i++) {
	       if (!strcmp(argv[i], "-x")) nx = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-y")) ny = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-u")) nu = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-d")) nd = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-l")) nl = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-r")) nr = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-p")) np = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-m")) sm = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-w")) {
	    w = 1 ;
	    xmin = atof(argv[++i]) ;
	    ymin = atof(argv[++i]) ;
	    xmax = atof(argv[++i]) ;
	    ymax = atof(argv[++i]) ;
	    }
	  else if (!strcmp(argv[i], "-h")) {
	    fprintf(stderr,
		" USAGE: point -x # -y # {-u # -d # -r # -l # -p # -m # -w # # # #}\n") ;
	    return(0) ;
	    }
	  }
	if (np < 0) {
	  printf("Mp %d\n", -np) ;
	  np = 0 ;
	  }
	if (sm < 0.0) {
	  printf("Ss %f\n", -sm) ;
	  printf("Sr 0.0\n") ;
	  printf("Ms %f\n", -sm/3.0) ;
	  nm = 0 ;
	  }
	else if (sm > 0.0) nm = (int)sm ;

	while (fgets(txt, 500, stdin) != NULL) {
	  x[0] = '\0' ;
	  y[0] = '\0' ;
	  u[0] = '\0' ;
	  d[0] = '\0' ;
	  l[0] = '\0' ;
	  r[0] = '\0' ;
	  p[0] = '\0' ;
	  m[0] = '\0' ;
	  j = -1 ;
	  n = 0 ;
	  for (i = 0; i < strlen(txt); i++) {
	    if (txt[i] == ' ') {
	      j = -1 ;
	      continue ;
	      }
	    if (j == -1) n++ ;
	    j++ ;
	    if (j > MAXL-2) j = MAXL-2 ;
	         if (n == nx) {x[j] = txt[i] ; x[j+1] = '\0' ; }
	    else if (n == ny) {y[j] = txt[i] ; y[j+1] = '\0' ; }
	    else if (n == nu) {u[j] = txt[i] ; u[j+1] = '\0' ; }
	    else if (n == nd) {d[j] = txt[i] ; d[j+1] = '\0' ; }
	    else if (n == nl) {l[j] = txt[i] ; l[j+1] = '\0' ; }
	    else if (n == nr) {r[j] = txt[i] ; r[j+1] = '\0' ; }
	    else if (n == np) {p[j] = txt[i] ; p[j+1] = '\0' ; }
	    else if (n == nm) {m[j] = txt[i] ; m[j+1] = '\0' ; }
	    }
	  rx = atof(x) ;
	  ry = atof(y) ;
	  if (w == 1 && (rx<xmin || rx>xmax || ry<ymin || ry>ymax)) continue ;
	  if (np != 0) {
	    mp = atoi(p) ;
	    if (mp != 0) printf("Mp %d\n", mp) ;
	    }
	  if (nm != 0) {
	    sm = atof(m) ;
	    if (sm != 0.0) printf("Ms %f\n", sm) ;
	    }
	  printf("m %s %s\n", x, y) ;
	  if (nu != 0) {
	    printf("Sv -0.6\n") ;
	    printf("Sh 0.5\n") ;
	    printf("s %s\n", u) ;
	    }
	  if (nd != 0) {
	    printf("Sv 1.5\n") ;
	    printf("Sh 0.5\n") ;
	    printf("s %s\n", d) ;
	    }
	  if (nl != 0) {
	    printf("Sv 0.5\n") ;
	    printf("Sh 1.2\n") ;
	    printf("s %s\n", l) ;
	    }
	  if (nr != 0) {
	    printf("Sv 0.5\n") ;
	    printf("Sh 0.2\n") ;
	    printf("s %s\n", r) ;
	    }
	  printf("p\n") ;
	  }
	return(0) ;
	}
