/*
*  Program name:             ppm_inc  ver. 2.0
*
*  Program function:         Included library for JKG-PPM bridge routine
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 15, 1990
*
*  Date of revision:         Dec 15, 1993 to use calloc by Y.Murata
*                            June 8, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "jk_ppm.h"
#include "jk_macro.h"

long	Pic=0, Resol=90, Xdots, Ydots ;
float	X0, Y0, Xscl, Yscl ;
short	Red, Green, Blue, RedB, GreenB, BlueB ;
long	PixelMode, CurrentColor ;

#include "inc_lib.c"

/* ------------------------------------------------- graphic init ----------- */
void	Resol_Set(bcolor, space, xstr, ystr, xwid, ywid, ndev)
long	bcolor ;
float	space ;
double	*xstr, *ystr, *xwid, *ywid ;
long	*ndev ;
{
	void	Plot_Start_Sub() ;

	Xscl = (float)Resol / 2.54 ;
	Yscl = Xscl ;
	if (Pic == 0) {
		if (Xdots == 0) Xdots = 21.0 * (float)Resol / 2.54 ;
		if (Ydots == 0) Ydots = 28.9 * (float)Resol / 2.54 ;
		}
	else if (Pic == 1) {
		if (Resol == 400) {
			Xdots = 3213 ;
			Ydots = 4583 ;
			}
		else if (Resol == 320) {
			Xdots = 2570 ;
			Ydots = 3666 ;
			}
		else if (Resol == 267) {
			Xdots = 2142 ;
			Ydots = 3055 ;
			}
		else if (Resol == 200) {
			Xdots = 1606 ;
			Ydots = 2291 ;
			}
		else if (Resol == 133) {
			Xdots = 1071 ;
			Ydots = 1528 ;
			}
		else {
			fprintf(stderr, " resol != 400,320,267,200,133.\n") ;
			exit(-1) ;
			}
		}
	else if (Pic == 2) {
		if (Resol == 400) {
			Xdots = 2244 ;
			Ydots = 3213 ;
			}
		else if (Resol == 320) {
			Xdots = 1795 ;
			Ydots = 2570 ;
			}
		else if (Resol == 267) {
			Xdots = 1496 ;
			Ydots = 2142 ;
			}
		else if (Resol == 200) {
			Xdots = 1122 ;
			Ydots = 1606 ;
			}
		else if (Resol == 133) {
			Xdots = 748 ;
			Ydots = 1071 ;
			}
		else {
			fprintf(stderr, " resol != 400,320,267,200,133.\n") ;
			exit(-1) ;
			}
		}
	else if (Pic == 3) {
		if (Resol == 400) {
			Xdots = 3431 ;
			Ydots = 4709 ;
			}
		else if (Resol == 320) {
			Xdots = 2745 ;
			Ydots = 3767 ;
			}
		else if (Resol == 267) {
			Xdots = 2288 ;
			Ydots = 3139 ;
			}
		else if (Resol == 200) {
			Xdots = 1716 ;
			Ydots = 2354 ;
			}
		else if (Resol == 133) {
			Xdots = 1144 ;
			Ydots = 1569 ;
			}
		else {
			fprintf(stderr, " resol != 400,320,267,200,133.\n") ;
			exit(-1) ;
			}
		}
	else if (Pic == 4) {
		if (Resol == 400) {
			Xdots = 2370 ;
			Ydots = 3431 ;
			}
		else if (Resol == 320) {
			Xdots = 1896 ;
			Ydots = 2745 ;
			}
		else if (Resol == 267) {
			Xdots = 1580 ;
			Ydots = 2288 ;
			}
		else if (Resol == 200) {
			Xdots = 1185 ;
			Ydots = 1716 ;
			}
		else if (Resol == 133) {
			Xdots = 790 ;
			Ydots = 1144 ;
			}
		else {
			fprintf(stderr, " resol != 400,320,267,200,133.\n") ;
			exit(-1) ;
			}
		}
	X0 = (float)Xdots * space ;
	Y0 = (float)Ydots * space ;
	fprintf(stderr, " X0, Y0 = %f %f\n", X0, Y0) ;
	*xstr = 0.0 ;
	*ystr = 0.0 ;
	*xwid = ((float)Xdots - 2.0 * X0) / Xscl ;
	*ywid = ((float)Ydots - 2.0 * Y0) / Yscl ;
	*ndev = 3 ;
	if (bcolor == 99999) {
		RedB   = 255 ;
		GreenB = 255 ;
		BlueB  = 255 ;
		}
	else if (bcolor > -1) {
		RedB   = Rgb[bcolor][0] ;
		GreenB = Rgb[bcolor][1] ;
		BlueB  = Rgb[bcolor][2] ;
		}
	else {
		RedB   =  -bcolor / 1000000 ;
		GreenB = (-bcolor / 1000) % 1000 ;
		BlueB  =  -bcolor % 1000 ;
		}
	Plot_Start_Sub() ;
	}
/* ------------------------------------------------- color table set -------- */
void	Color_Default_Set()
{
	Ncolor = 8 ;
	Rgb[0][0] =   0 ; Rgb[1][1] =   0 ; Rgb[1][2] =   0 ;
	Rgb[1][0] = 255 ; Rgb[0][1] = 255 ; Rgb[0][2] = 255 ;
	Rgb[2][0] = 255 ; Rgb[2][1] =   0 ; Rgb[2][2] =   0 ;
	Rgb[3][0] =   0 ; Rgb[3][1] = 255 ; Rgb[3][2] =   0 ;
	Rgb[4][0] =   0 ; Rgb[4][1] =   0 ; Rgb[4][2] = 255 ;
	Rgb[5][0] =   0 ; Rgb[5][1] = 255 ; Rgb[5][2] = 255 ;
	Rgb[6][0] = 255 ; Rgb[6][1] =   0 ; Rgb[6][2] = 255 ;
	Rgb[7][0] = 255 ; Rgb[7][1] = 255 ; Rgb[7][2] =   0 ;
	}
/* ------------------------------------------------- color table set -------- */
void	Color_Table_Set()
{
	}
/* ------------------------------------------------------- color set -------- */
void	Color_Set(color)
long	color ;
{
	CurrentColor = color ;
	if (CurrentColor <= 0) {
		Red   =  -CurrentColor / 1000000 ;
		Green = (-CurrentColor / 1000) % 1000 ;
		Blue  =  -CurrentColor % 1000 ;
		}
	else if (CurrentColor < MAXC) {
		Red   = Rgb[CurrentColor][0] ;
		Green = Rgb[CurrentColor][1] ;
		Blue  = Rgb[CurrentColor][2] ;
		}
	}
/* -------------------------------------------------------------------------- */
void	backcolor()
{
	Red   = RedB ;
	Green = GreenB ;
	Blue  = BlueB ;
	}
/* -------------------------------------------------------------------------- */
void	forecolor()
{
	void	Color_Set() ;
	Color_Set(CurrentColor) ;
	}
