/*
*  Program name:             rotate  ver. 2.0
*
*  Program function:         JKG data rotation
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            October 14, 1993
*
*  Date of revision:         June 7, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. rotation degree
*                            2. rotation center X (default: 0)
*                            3. rotation center Y (default: 0)
*
*  Input:                    JKG data (from stdin)
*
*  Output:                   converted JKG data (to stdout)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	r, x0, y0 ;
	double	x, y, x2, y2, xo, yo, cos(), sin(), cosr, sinr, atof() ;
	char	hed, txt[200] ;

	if (argc < 2 || strcmp(argv[1], "-h") == 0 || (r=atof(argv[1])) == 0) {
	  fprintf(stderr, " USAGE: rotate {angle}\n") ;
	  return(0) ;
	  }
	if (argc > 2) x0 = atof(argv[2]) ; else x0 = 0.0 ;
	if (argc > 3) y0 = atof(argv[3]) ; else y0 = 0.0 ;
	r = r * 1.745329251993889e-2 ;
	cosr = cos(r) ;
	sinr = sin(r) ;

	while ((fgets(txt, 200, stdin)) != NULL) {
	  if (txt[0] == 'l' || txt[0] == 'm') {
	    if (sscanf(txt, "%c %lf %lf", &hed,&x,&y) != 3) {
	        fprintf(stderr, " JKG l/m data error.\n") ;
	        break ;
	      }
	    xo = (x - x0) * cosr - (y - y0) * sinr + x0 ;
	    yo = (x - x0) * sinr + (y - y0) * cosr + y0 ;
	    printf("%c %lf %lf\n", hed, xo, yo) ;
	    }
	  else if(txt[0] == 'A' || txt[0] == 'W') {
	    if (sscanf(txt, "%c %lf %lf %lf %lf", &hed,&x,&y,&x2,&y2) != 5) {
	      fprintf(stderr, " JKG A-W data error.\n") ;
	      break ;
	      }
	    xo = (x - x0) * cosr - (y - y0) * sinr + x0 ;
	    yo = (x - x0) * sinr + (y - y0) * cosr + y0 ;
	    printf("%c %lf %lf", hed, xo, yo) ;
	    xo = (x2 - x0) * cosr - (y2 - y0) * sinr + x0 ;
	    yo = (x2 - x0) * sinr + (y2 - y0) * cosr + y0 ;
	    printf(" %lf %lf\n", xo, yo) ;
	    }
	  else printf("%s", txt) ;
	  }
	return(0) ;
	}
