/*
*  Program name:             second  ver. 2.0
*
*  Program function:         degree <--> second  convert
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            December 26, 1988
*
*  Date of revision:         January 11, 1993 by  Y.Murata
*                            June 5, 1995  for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -i60  inputs  are 100.60 mode ( Default: 100.100 )
*                            -o60  outputs are 100.60 mode ( Default: 100.100 )
*                            +90   add 90 degrees
*                            -90   sub 90 degrees
*                            -ds   degree -> second ( Default )
*                            -sd   second -> degree
*                            -ms   minute -> second
*                            -sm   second -> minute
*                            -dm   degree -> minute
*                            -md   minute -> degree
*
*  Input:                    JKG from standard input
*
*  Output:                   JKG to standard output
*/

#include <stdio.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, from, to, in, out ;
	char	hed, txt[500] ;
	double	dx, dy, dx2, dy2, add ;
	void	conv() ;

	if (argc > 1 && strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr, " USAGE: second {-i60 -o60} [+90 -90] ") ;
	  fprintf(stderr, "[-ds -sd -ms -sm -dm -md]\n") ;
	  return(0) ;
	  }
	from = 1 ;
	to = 3 ;
	in = 0 ;
	out = 0 ;
	add = 0.0 ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-i60")== 0) in  = 1 ;
	  else if (strcmp(argv[i], "-o60")== 0) out = 1 ;
	  else if (strcmp(argv[i], "-90") == 0) add = -1.0 ;
	  else if (strcmp(argv[i], "+90") == 0) add =  1.0 ;
	       if (strcmp(argv[i], "-ds") == 0) from = 1 ;
	  else if (strcmp(argv[i], "-sd") == 0) from = 3 ;
	  else if (strcmp(argv[i], "-ms") == 0) from = 2 ;
	  else if (strcmp(argv[i], "-sm") == 0) from = 3 ;
	  else if (strcmp(argv[i], "-dm") == 0) from = 1 ;
	  else if (strcmp(argv[i], "-md") == 0) from = 2 ;
	  else if (strcmp(argv[i], "-dd") == 0) from = 1 ;
	  else if (strcmp(argv[i], "-mm") == 0) from = 2 ;
	  else if (strcmp(argv[i], "-ss") == 0) from = 3 ;
	       if (strcmp(argv[i], "-ds") == 0) to = 3 ;
	  else if (strcmp(argv[i], "-sd") == 0) to = 1 ;
	  else if (strcmp(argv[i], "-ms") == 0) to = 3 ;
	  else if (strcmp(argv[i], "-sm") == 0) to = 2 ;
	  else if (strcmp(argv[i], "-dm") == 0) to = 2 ;
	  else if (strcmp(argv[i], "-md") == 0) to = 1 ;
	  else if (strcmp(argv[i], "-dd") == 0) to = 1 ;
	  else if (strcmp(argv[i], "-mm") == 0) to = 2 ;
	  else if (strcmp(argv[i], "-ss") == 0) to = 3 ;
	  }

	while ((fgets(txt, 500, stdin)) != NULL) {
	  if (txt[0] == 'l' || txt[0] == 'm') {
	    sscanf(txt, "%c %lf %lf", &hed, &dx, &dy) ;
	    conv(&dx, &dy, in, out, from, to, add) ;
	    printf("%c %lf %lf\n", txt[0], dx, dy) ;
	    }
	  else if (txt[0] == 'A' || txt[0] == 'W') {
	    sscanf(txt, "%c %lf %lf %lf %lf", &hed, &dx, &dy, &dx2, &dy2) ;
	    conv(&dx, &dy, in, out, from, to, add) ;
	    printf("%c %lf %lf ", txt[0], dx, dy) ;
	    conv(&dx2, &dy2, in, out, from, to, add) ;
	    printf("%lf %lf\n", dx2, dy2) ;
	    }
	  else printf("%s", txt) ;
	  }
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	conv(dx, dy, in, out, from, to, add)
double	*dx, *dy, add ;
int	in, out, from, to ;
{
	int	ix, iy ;

	if (in == 1) {
	  ix = (int)(*dx) ;
	  iy = (int)(*dy) ;
	  *dx = (double)ix + (*dx - (double)ix) / 0.6 ;
	  *dy = (double)iy + (*dy - (double)iy) / 0.6 ;
	  }
	if (from == 1) {
	  *dx = *dx * 3600.0 ;
	  *dy = *dy * 3600.0 + add * 324000.0 ;
	  }
	else if (from == 2) {
	  *dx = *dx * 60.0 ;
	  *dy = *dy * 60.0 + add * 324000.0 ;
	  }
	else {
	  *dy = *dy + add * 324000.0 ;
	  }
	if (to == 1) {
	  *dx = *dx / 3600.0 ;
	  *dy = *dy / 3600.0 ;
	  }
	else if (to == 2) {
	  *dx = *dx / 60.0 ;
	  *dy = *dy / 60.0 ;
	  }
	if (out == 1) {
	  ix = (int)(*dx) ;
	  iy = (int)(*dy) ;
	  *dx = (double)ix + (*dx - (double)ix) * 0.6 ;
	  *dy = (double)iy + (*dy - (double)iy) * 0.6 ;
	  }
	}
