/*
*  Program name:             sinuso  ver. 2.0
*
*  Program function:         Sinusoldal map projection (sphere), forward routine
*                            (lat, lon) ==>> (x, y)
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 10, 1990
*
*  Date of revision:         June 5, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. center longitude (unit:degree)
*                            2. GRS80 (default: Bessel)
*
*  Input:                    JKG (unit:second) from stdin
*                                 project only 'l' 'm' 'A' 'W' data
*
*  Output:                   projected JKG (unit:meter) to stdout
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

double	pi, r, b3600, lono, lon0 ;
int	ip, grs ;

main(argc, argv)
int	argc ;
char	*argv[] ;

{
	char	hed, txt[500] ;
	double	lat, lon, lat2, lon2, x, y ;
	void	sinuso() ;
/* ---- argument read ---- */
	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr, " USAGE: sinuso {center-lon}\n") ;
	  return(0) ;
	  }
	if (argc >= 2) sscanf(argv[1], "%lf", &lon0) ; else lon0 = 136.0 ;
	if ( lon0 <   0.0 ) lon0 = lon0 + 360.0 ;
	if ( lon0 > 360.0 ) lon0 = lon0 - 360.0 ;
	grs = 0 ;
	if (argc >= 3 && strcmp("GRS80", argv[3])==0) grs = 1 ;
	if (argc >= 3 && strcmp("grs80", argv[3])==0) grs = 1 ;
	if (argc >= 3 && strcmp("grs", argv[3])==0) grs = 1 ;
/* ---- parameter set ---- */
	pi = 1.745329251993889e-2 ;
	if (grs == 0) r = pi * 6377397.155 ;
	else r = pi * 6378137.000 ;
	b3600 = 1.0 / 3600.0 ;
	ip = 0 ;
	lono = -999999.0 ;
/* ---- data read roop ---- */
	while ( (fgets(txt, 500, stdin)) != NULL ) {
	  if (txt[0] == 'l' || txt[0] == 'm') {
	    if (sscanf(txt, "%c %lf %lf", &hed, &lon, &lat) != 3) {
	      fprintf(stderr, " JKG l/m data error.\n") ;
	      return(-1) ;
	      }
	    sinuso(&lon, &lat, &x, &y) ;
	    if (ip == 3) hed = 'm' ;
	    printf("%c %11.2lf %11.2lf\n", hed, x, y) ;
	    }
	  else if (txt[0] == 'A' || txt[0] == 'W') {
	    if (sscanf(txt, "%c %lf %lf %lf %lf",
	      &hed, &lon, &lat, &lon2, &lat2) != 5) {
	      fprintf(stderr, " JKG A/W data error.\n") ;
	      return(-1) ;
	      }
	    sinuso(&lon, &lat, &x, &y) ;
	    printf("%c %11.2lf %11.2lf ", hed, x, y) ;
	    sinuso(&lon2, &lat2, &x, &y) ;
	    printf("%11.2lf %11.2lf\n", x, y) ;
	    ip = 0 ;
	    }
	  else printf("%s", txt) ;
	  }
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	sinuso(lon, lat, x, y)
double	*lon, *lat, *x, *y ;
{
	double	fabs() ;

	*lat = *lat * b3600 ;
	*lon = *lon * b3600 - lon0 ;
	if (*lon < -180.0) *lon = *lon + 360.0 ;
	else if(*lon > 180.0) *lon = *lon - 360.0 ;
	if (fabs(*lon-lono) > 300.0) ip = 3 ;
	else ip = 2 ;
	*x = r * (*lon) * cos(*lat * pi) ;
	*y = r * (*lat) ;
	lono = *lon ;
	}
