/*
*  Program name:             window  ver. 2
*
*  Program function:         JKG format data window clipping
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            May 30, 1991
*
*  Date of revision:         Feb 5, 2000 Add area clipping by Y. Murata.
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1: window lower-left  x
*                            2: window lower-left  y
*                            3: window upper-right y
*                            4: window upper-right y
*                              or -f limit file (m xmin ymin m xmin ymin)
*
*  Input:                    JKG format data from standard input
*
*  Output:                   Clipped JKG format data to standard output
*
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#define	ACCY	0.000001
#define	MaxD	100000

double	Xmin, Ymin, Xmax, Ymax ;

int	N=0 ;

char	Dhed[MaxD] ;
double	Dx[MaxD], Dy[MaxD] ;

int	Old_d = -1 ;
double	Old_x, Old_y ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	FILE	*fp ;
	char	hed, txt[500] ;
	double	x, y, atof() ;
	int	area=0 ;
	void	SegClip(), AreaClip() ;

	if (argc == 5 && strcmp(argv[1], "-f") != 0) {
	  Xmin = atof(argv[1]) ;
	  Ymin = atof(argv[2]) ;
	  Xmax = atof(argv[3]) ;
	  Ymax = atof(argv[4]) ;
	  }

	else if (argc == 3) {
	  fp = fopen(argv[2], "r") ;
	  fscanf(fp, "%c %lf %lf %lf %lf\n", &hed, &Xmin, &Ymin, &Xmax, &Ymax) ;
	  fclose(fp) ;
	  }
	else {
	  fprintf(stderr, "Usage: window xmin ymin xmax ymax < in > out\n") ;
	  fprintf(stderr, "   or  window -f limit-file < in > out %d\n", argc) ;
	  return(0) ;
	  }

/*	fprintf(stderr, " lower-left  = %lf %lf\n", Xmin, Ymin) ;	*/
/*	fprintf(stderr, " upper-right = %lf %lf\n", Xmax, Ymax) ;	*/

	while ( (fgets(txt, 500, stdin)) != NULL ) {
	  if ( txt[0] == 'l' || txt[0] == 'm' ) {
	    sscanf(txt, "%c %lf %lf", &hed, &x, &y) ;
	    if (area == 1) {
	      Dhed[N] = hed ;
	      Dx[N] = x ;
	      Dy[N] = y ;
	      N++ ;
	      if (N > MaxD) {
	        fprintf(stderr, "N > MaxD\n") ;
	        exit(-1) ;
	        }
	      continue ;
	      }
	    else {
	      SegClip(hed, x, y) ;
	      }
	    }
	  else if (txt[0] == 'r' && (txt[1] == 'm' || txt[1] == 'l')) {
	    if (Old_d == 4) printf("%s", txt) ;
	    }
	  else if (txt[0] == 'p' || txt[0] == 's') {
	    if (Old_d == 4) printf("%s", txt) ;
	    }
	  else if (txt[0] == 'n') {
	    area = 1 ;
	    N = 0 ;
	    }
	  else if (txt[0] == 'f') {
	    if (area == 0 && N > 0) printf("%s", txt) ;
	    else {
	      if (Dx[0] != Dx[N-1] || Dy[0] != Dy[N-1]) {
	        Dhed[N] = 'm' ;
	        Dx[N] = Dx[0] ;
	        Dy[N] = Dy[0] ;
	        N++ ;
		if (N > MaxD) {
			fprintf(stderr, "N > MaxD\n") ;
			exit(-1) ;
			}
	        }
	      AreaClip() ;
	      area = 0 ;
	      }
	    }
	  else printf("%s", txt) ;
	  }
	return(0) ;
	} 

/* -------------------------------------------------------------------------- */
void	SegClip(hed, x, y)
char	hed ;
double	x, y ;
{
	int	d, Dim(), Cross2() ;
	int	dd[9][2] = {{1, 3}, {0, 2}, {1, 5}, {0, 6},
		            {4, 4}, {2, 8}, {3, 7}, {6, 8}, {5, 7}} ;
	double	xr1, yr1, xr2, yr2 ;
	void	Cross1() ;

	d = Dim(x, y) ;
	if (d == 4) {
	  if (Old_d == 4) {
	    printf("%c %lf %lf\n", hed, x, y) ;
	    }
	  else {
	    if (hed == 'l') {
	      Cross1(Old_x, Old_y, x, y, &xr1, &yr1) ;
	      printf("m %lf %lf\n", xr1, yr1) ;
	      }
	    printf("%c %lf %lf\n", hed, x, y) ;
	    }
	  }
	else if (Old_d == 4) {
	  if (hed == 'l') {
	    Cross1(Old_x, Old_y, x, y, &xr1, &yr1) ;
	    printf("%c %lf %lf\n", hed, xr1, yr1) ;
	    }
	  }
	else if (Old_d != -1 && Old_d != dd[d][0] && Old_d != dd[d][1]) {
	  if (hed == 'l') {
	    if (Cross2(Old_x, Old_y, x, y, &xr1, &yr1, &xr2, &yr2) == 1) {
	      printf("%c %lf %lf\n", 'm', xr1, yr1) ;
	      printf("%c %lf %lf\n", 'l', xr2, yr2) ;
	      }
	    }
	  }
	Old_x = x ;
	Old_y = y ;
	Old_d = d ;
	}

/* -------------------------------------------------------------------------- */
void	AreaClip()
{
	int	i ;
	double	xminw, yminw, xmaxw, ymaxw, w ;
	void	VertexClip() ;

	xminw = Dx[0] ;
	yminw = Dy[0] ;
	xmaxw = Dx[0] ;
	ymaxw = Dy[0] ;
	for (i = 1; i < N; i++) {
	  if (xminw > Dx[i]) xminw = Dx[i] ;
	  if (yminw > Dy[i]) yminw = Dy[i] ;
	  if (xmaxw < Dx[i]) xmaxw = Dx[i] ;
	  if (ymaxw < Dy[i]) ymaxw = Dy[i] ;
	  }
	if (xmaxw<Xmin || xminw>Xmax || ymaxw<Ymin || yminw>Ymax) return ;
	if (xmaxw > Xmax || xminw < Xmin || ymaxw > Ymax || yminw < Ymin) {
	  VertexClip(Ymin) ;
	  for (i = 0; i < N; i++) Dy[i] = -Dy[i] ;
	  VertexClip(-Ymax) ;
	  for (i = 0; i < N; i++) {
	    w = Dx[i] ;
	    Dx[i] = -Dy[i] ;
	    Dy[i] = w ;
	    }
	  VertexClip(Xmin) ;
	  for (i = 0; i < N; i++) Dy[i] = -Dy[i] ;
	  VertexClip(-Xmax) ;
	  for (i = 0; i < N; i++) {
	    w = Dx[i] ;
	    Dx[i] = -Dy[i] ;
	    Dy[i] = w ;
	    }
	  }
	if (N > 0) {
	  printf("n\n") ;
	  for (i = 0; i < N; i++) 
	    printf("%c %lf %lf\n", Dhed[i], Dx[i], Dy[i]) ;
	  printf("f\n") ;
	  }
	}

/* -------------------------------------------------------------------------- */
void	VertexClip(y)
double	y ;
{
	int	i, nw ;
	char	dhedw[MaxD] ;
	double	dxw[MaxD], dyw[MaxD] ;

	nw = 0 ;
	for (i = 0; i < N; i++) {
	  if (Dy[i] >= y) {
	    if (i > 0 && Dy[i-1] < y) {
	      dhedw[nw] = 'm' ;
	      dxw[nw] = (y-Dy[i-1])/(Dy[i]-Dy[i-1])*(Dx[i]-Dx[i-1])+Dx[i-1] ;
	      dyw[nw] = y ;
	      nw++ ;
	      }
	    dhedw[nw] = Dhed[i] ;
	    dxw[nw] = Dx[i] ;
	    dyw[nw] = Dy[i] ;
	    nw++ ;
	    }
	  else if (i > 0 && Dy[i-1] >= y) {
	    dhedw[nw] = Dhed[i] ;
	    dxw[nw] = (y-Dy[i-1])/(Dy[i]-Dy[i-1])*(Dx[i]-Dx[i-1])+Dx[i-1] ;
	    dyw[nw] = y ;
	    nw++ ;
	    }
	  }
	if (Dy[0] < y && nw > 0) {
	  dhedw[nw] = 'm' ;
	  dxw[nw] = dxw[0] ;
	  dyw[nw] = dyw[0] ;
	  nw++ ;	  
	  }
	for (i = 0; i < nw; i++) {
	  Dhed[i] = dhedw[i] ;
	  Dx[i] = dxw[i] ;
	  Dy[i] = dyw[i] ;
	  }
	N = nw ;
	}

/* -------------------------------------------------------------------------- */
int Dim(x, y)
double	x, y ;
{
	int	dx, dy, d ;

	if (y < Ymin) dy = 0 ;
	else if (y <= Ymax) dy = 1 ;
	else dy = 2 ;
	if (x < Xmin) dx = 0 ;
	else if (x <= Xmax) dx = 1 ;
	else dx = 2 ;
	d = dy * 3 + dx ;
	return(d) ;
	}

/* -------------------------------------------------------------------------- */
void	Cross1(x1, y1, x2, y2, xr, yr)
double	x1, y1, x2, y2, *xr, *yr ;
{
	double	x, y ;
	int	Cross() ;

	if (Cross(x1, y1, x2, y2, Xmin, Ymin, Xmax, Ymin, &x, &y) != 1)
	if (Cross(x1, y1, x2, y2, Xmin, Ymax, Xmax, Ymax, &x, &y) != 1)
	if (Cross(x1, y1, x2, y2, Xmin, Ymin, Xmin, Ymax, &x, &y) != 1)
	if (Cross(x1, y1, x2, y2, Xmax, Ymin, Xmax, Ymax, &x, &y) != 1) {
	  fprintf(stderr, "%lf %lf %lf %lf\n", x1, y1, x2, y2) ;
	  fprintf(stderr, "Cross 1 error\n") ;
	  exit(-1) ;
	  }
	*xr = x ;
	*yr = y ;
	}

/* -------------------------------------------------------------------------- */
int	Cross2(x1, y1, x2, y2, xr1, yr1, xr2, yr2)
double	x1, y1, x2, y2, *xr1, *yr1, *xr2, *yr2 ;
{
	double	x, y, xw[4], yw[4], w1, w2 ;
	int	i, Cross() ;

	i = 0 ;
	if (Cross(x1, y1, x2, y2, Xmin, Ymin, Xmax, Ymin, &x, &y) == 1) {
	  xw[i] = x ;
	  yw[i] = y ;
	  i++ ;
	  }
	if (Cross(x1, y1, x2, y2, Xmin, Ymax, Xmax, Ymax, &x, &y) == 1) {
	  xw[i] = x ;
	  yw[i] = y ;
	  i++ ;
	  }
	if (Cross(x1, y1, x2, y2, Xmin, Ymin, Xmin, Ymax, &x, &y) == 1) {
	  xw[i] = x ;
	  yw[i] = y ;
	  i++ ;
	  }
	if (Cross(x1, y1, x2, y2, Xmax, Ymin, Xmax, Ymax, &x, &y) == 1) {
	  xw[i] = x ;
	  yw[i] = y ;
	  i++ ;
	  }
	if (i > 0 && i != 2) {
	  fprintf(stderr, "# %lf %lf %lf %lf\n", x1, y1, x2, y2) ;
	  fprintf(stderr, "Cross 2 error\n") ;
	  exit(-1) ;
	  }
	if (i == 0) return(0) ;
	w1 = (xw[0] - x1) * (xw[0] - x1) + (yw[0] - y1) * (yw[0] - y1) ;
	w2 = (xw[1] - x1) * (xw[1] - x1) + (yw[1] - y1) * (yw[1] - y1) ;
	if (w1 < w2) {
	  *xr1 = xw[0] ;
	  *yr1 = yw[0] ;
	  *xr2 = xw[1] ;
	  *yr2 = yw[1] ;
	  }
	else {
	  *xr1 = xw[1] ;
	  *yr1 = yw[1] ;
	  *xr2 = xw[0] ;
	  *yr2 = yw[0] ;
	  }
	return(1) ;
	}

/* -------------------------------------------------------------------------- */
int	Cross(xk, yk, xl, yl, xm, ym, xn, yn, x, y)
double	xk, yk, xl, yl, xm, ym, xn, yn, *x, *y ;
{
	double	xlk, ylk, xnm, ynm, xmk, ymk, s, t, det, detinv ;

	xlk = xl - xk ;
	ylk = yl - yk ;
	xnm = xn - xm ;
	ynm = yn - ym ;
	xmk = xm - xk ;
	ymk = ym - yk ;
	det = xnm * ylk - ynm * xlk ;
	if (fabs(det) < ACCY) return(0) ;
	detinv = 1.0 / det ;
	s = (xnm * ymk - ynm * xmk) * detinv ;
	t = (xlk * ymk - ylk * xmk) * detinv ;
	if (s < 0.0 || s > 1.0 || t < 0.0 || t > 1.0) return(0) ;
	*x = xk + xlk * s ;
	*y = yk + ylk * s ;
	return(1) ;
	}
