/*
*  Program name:             x11_lib  ver. 1.0
*
*  Program function:         JKG-X11 bridge routine
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         May 30, 1995 by Y. Murata (rewrite by C)
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -x dots  x-direction window size
*                            -y dots  y-direction window size
*                            -b ratio ratio of the space (default 0.05)
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<X11/Xlib.h>
#include	<X11/X.h>
#include	<X11/Xutil.h>
#include	<X11/cursorfont.h>
#include	<X11/Xatom.h>
#include	"jk_macro.h"

#define	XDOTS_MAX	1600
#define	YDOTS_MAX	1200

#define	WHITE	0
#define	BLACK	1
#define	MAXP	1025
#define	MAXC	256
#define	Xscl	37.0
#define	Yscl	37.0

Display		*Dis ;
Window		Win, Root ;
Colormap	Cmap ;
XColor		Acolor[MAXC] ;
GC		Gc ;
Visual		*vis ;

unsigned int	Xdots, Ydots ;
float	X0, Y0 ;

unsigned long	Pl[2], Px[256], PxT ;
long	CMode, CurrentColor ;
int	MaxY, Scr ;

#include "inc_lib.c"

/* ------------------------------------------------- graphic init ----------- */
void	plotst_(xstr, ystr, xwid, ywid, ndev, argc, argv)
double	*xstr, *ystr, *xwid, *ywid ;
long	*ndev ;
int	argc ;
char	*argv[] ;
{
	long	i, white, black ;
	void	Plot_Start_Sub() ;
	int	x, y ;
	unsigned int	border, depth ;
	float	space ;

	Xdots = 666 ;
	Ydots = 963 ;
	Xdots = 1024 ;
	Ydots = 768 ;
	space = 0.05 ;
	for (i = 0; i < argc; i++) {
	  if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr, " USAGE: %s {-s # -r # -c $ -p $ $ ", argv[0]) ;
	    fprintf(stderr, "-x # -y # -b #}\n") ;
	    exit(0) ;
	    }
	  else if (strcmp(argv[i], "-x") == 0) Xdots = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-y") == 0) Ydots = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-b") == 0) space = atof(argv[++i]) ;
	  }

	if (Xdots > XDOTS_MAX) Xdots = XDOTS_MAX ;
	if (Ydots > YDOTS_MAX) Ydots = YDOTS_MAX ;

	Dis = XOpenDisplay(NULL) ;
	Scr = DefaultScreen(Dis) ;
	vis = DefaultVisual(Dis, Scr) ;
	CMode = vis->class ;
	white = WhitePixel(Dis, Scr) ;
	black = BlackPixel(Dis, Scr) ;
	Win = XCreateSimpleWindow(Dis, RootWindow(Dis, 0), 5, 5,
		Xdots+5, Ydots+5, 2, 1, white) ;
	XMapWindow(Dis, Win) ;
	Gc = XCreateGC(Dis, Win, 0, 0) ;
	XSetLineAttributes(Dis, Gc, 1, LineSolid, CapButt, JoinMiter) ;
	XSetForeground(Dis, Gc, black) ;
	XSetBackground(Dis, Gc, white) ;
	XFlush(Dis) ;
	sleep(3) ;
	XGetGeometry(Dis, Win, &Root, &x, &y, &Xdots, &Ydots, &border, &depth) ;

	X0 = (float)Xdots * space ;
	Y0 = (float)Ydots * space ;
	*xstr = 0.0 ;
	*ystr = 0.0 ;
	*xwid = ((float)Xdots - 2.0 * X0) / Xscl ;
	*ywid = ((float)Ydots - 2.0 * Y0) / Yscl ;
	*ndev = 1 ;
	Plot_Start_Sub() ;
	MaxY = Ydots ;
	}
/* ------------------------------------------------- graphic close ---------- */
void	ploten_()
{
	FILE	*fp ;
/*	Font	fnt ; */
	XEvent	evt ;
	Cursor	cs ;
	int	ix, iy, ixo, iyo ;
	double	xx, yy ;
	int	obutton, oline, ip ;
	char	ichar[2] ;
	void	conv_r() ;

	fprintf(stderr, "[Plot-End X11]\n") ;

	obutton = 0 ;
	oline = 0 ;
	ip = 0 ;
	ichar[0] = 'a' - 1 ;
	ichar[1] = '\0' ;
	XSelectInput(Dis, Win, ButtonPressMask | ButtonReleaseMask) ;
/*	fnt = XLoadFont(Dis, "fg-40") ; */
/*	XSetFont(Dis, Gc, fnt) ; */

	cs = XCreateFontCursor(Dis, XC_dotbox) ;
	XDefineCursor(Dis, Win, cs) ;
	XFlush(Dis) ;

	while (1) {
	  XNextEvent(Dis, &evt) ;
	  if (evt.type != ButtonPress) continue ;
	  switch (evt.xbutton.button) {
	    case Button1 :
	      ichar[0] = ichar[0] + 1 ;
	      if (ichar[0] > 'z') ichar[0] = 'a' ;
	      xx = evt.xbutton.x ;
	      yy = MaxY - evt.xbutton.y ;
	      conv_r(&xx, &yy) ;
	      if (ip == 0) {
	        fp = fopen("x11.output", "w") ;
	        ip = 1 ;
	        }
	      fprintf(fp, "m %12.2f %12.2f\n", xx, yy) ;
	      fprintf(fp, "c %c\n", ichar[0]) ;
/*	      XDrawString(Dis, Win, Gc, ix, iy, ichar, 1) ; */
	      obutton = 0 ;
	      oline = 0 ;
	      break ;
	    case Button2 :
	      if (obutton == 1) {
	        XCloseDisplay(Dis) ;
	        if (ip == 1) fclose(fp) ;
	        exit(0) ;
	        }
	      obutton = 1 ;
	      oline = 0 ;
	      break ;
	    case Button3 :
	      xx = evt.xbutton.x ;
	      yy = MaxY - evt.xbutton.y ;
	      conv_r(&xx, &yy) ;
	      if (ip == 0) {
	        fp = fopen("x11.output", "w") ;
	        ip = 1 ;
	        }
	      if (oline == 0)
	        fprintf(fp,"m %12.2lf %12.2lf\n",xx,yy);
	      else {
	        fprintf(fp, "l %12.2lf %12.2lf\n", xx, yy) ;
	        XDrawLine(Dis, Win, Gc, ixo, iyo, ix, iy) ;
	        XFlush(Dis) ;
	        }
	      ixo = ix ;
	      iyo = iy ;
	      obutton = 0 ;
	      oline = 1 ;
	    }
	  }
	}
/* ------------------------------------------------- color table set -------- */
void	Color_Default_Set()
{
	int	i ;

	Cmap = DefaultColormap(Dis, 0) ;
        Ncolor = 8 ;
	if (CMode == TrueColor) {
          Rgb[0][0] =   0 ; Rgb[0][1] =   0 ; Rgb[0][2] =   0 ;
          Rgb[1][0] = 255 ; Rgb[1][1] = 255 ; Rgb[1][2] = 255 ;
          Rgb[2][0] = 255 ; Rgb[2][1] =   0 ; Rgb[2][2] =   0 ;
          Rgb[3][0] =   0 ; Rgb[3][1] = 255 ; Rgb[3][2] =   0 ;
          Rgb[4][0] =   0 ; Rgb[4][1] =   0 ; Rgb[4][2] = 255 ;
          Rgb[5][0] =   0 ; Rgb[5][1] = 255 ; Rgb[5][2] = 255 ;
          Rgb[6][0] = 255 ; Rgb[6][1] =   0 ; Rgb[6][2] = 255 ;
          Rgb[7][0] = 255 ; Rgb[7][1] = 255 ; Rgb[7][2] =   0 ;
	  for (i = 8; i < MAXC; i++) {
            Rgb[i][0] = 150 ; Rgb[i][1] = 150 ; Rgb[i][2] = 150 ;
	    }
	  }
	else {
	  for (i = 0; i < 256; i++) Px[i] = i ;
	  XAllocNamedColor(Dis, Cmap, "white",   &Acolor[0], &Acolor[1]) ;
	  Px[0] = Acolor[0].pixel ;
	  XAllocNamedColor(Dis, Cmap, "black",   &Acolor[0], &Acolor[1]) ;
	  Px[1] = Acolor[0].pixel ;
	  XAllocNamedColor(Dis, Cmap, "red",     &Acolor[0], &Acolor[1]) ;
	  Px[2] = Acolor[0].pixel ;
	  XAllocNamedColor(Dis, Cmap, "green",   &Acolor[0], &Acolor[1]) ;
	  Px[3] = Acolor[0].pixel ;
	  XAllocNamedColor(Dis, Cmap, "blue",    &Acolor[0], &Acolor[1]) ;
	  Px[4] = Acolor[0].pixel ;
	  XAllocNamedColor(Dis, Cmap, "cyan",    &Acolor[0], &Acolor[1]) ;
	  Px[5] = Acolor[0].pixel ;
	  XAllocNamedColor(Dis, Cmap, "magenta", &Acolor[0], &Acolor[1]) ;
	  Px[6] = Acolor[0].pixel ;
	  XAllocNamedColor(Dis, Cmap, "yellow",  &Acolor[0], &Acolor[1]) ;
	  Px[7] = Acolor[0].pixel ;
	  }
	}
/* ------------------------------------------------- color table set -------- */
void	Color_Table_Set()
{
	int	i ;
	int	flags = DoRed | DoGreen | DoBlue ;

	if (Ncolor > 256) {
	  fprintf(stderr, " Number of colors > 256\n") ;
	  exit(0) ;
	  }
	if (Ncolor < 2) Ncolor = 2 ;
	Cmap = DefaultColormap(Dis, 0) ;

	if (CMode != TrueColor) {
	  if (XAllocColorCells(Dis, Cmap, True, Pl, 0, Px, Ncolor)) {
	    for (i = 0; i < Ncolor; i++) {
	      Acolor[i].pixel = Px[i] ;
	      Acolor[i].flags = flags ;
	      Acolor[i].red   = Rgb[i][0] * 256 ;
	      Acolor[i].green = Rgb[i][1] * 256 ;
	      Acolor[i].blue  = Rgb[i][2] * 256 ;
	      }
	    XStoreColors(Dis, Cmap, Acolor, Ncolor) ;
	    XFlush(Dis) ;
	    }
	  }
	}
/* ------------------------------------------------------- color set -------- */
void	Color_Set(icolor)
int	icolor ;
{
	int	c, r, g, b ;

	if (CurrentColor != icolor) {
	  CurrentColor = icolor ;
	  c = icolor ;
	  if (c < -255255255) c = -255255255 ;
	  if (c > MAXC-1) c = MAXC-1 ;
	  if (CMode == TrueColor) {
	    if (c <= 0) {
	      r =  -c / 1000000 ;
	      g = (-c / 1000) % 1000 ;
	      b =  -c % 1000 ;
	      PxT = r*256*256 + g*256 + b ;
	      }
	    else  {
	      PxT = Rgb[c][0]*256*256 +
	            Rgb[c][1]*256 +
	            Rgb[c][2] ;
	      }
	    XSetForeground(Dis, Gc, PxT) ;
	    }
	  else if (c > -1 && c < Ncolor) {
	    printf("Not TrueColor %d %d\n", c, Px[c]) ;
	    XSetForeground(Dis, Gc, Px[c]) ;
	    }
	  }
	}
/* ------------------------------------------------- pixel set -------------- */
void	pixel(x, y)
int	x, y ;
{
	if (x >= Ixmin && x <= Ixmax && y >= Iymin && y <= Iymax)
		XDrawPoint(Dis, Win, Gc, x, MaxY-y) ;
	}
/* -------------------------------------------------------------------------- */
void	backcolor()
{
	XSetForeground(Dis, Gc, WHITE) ;
	}
/* -------------------------------------------------------------------------- */
void	forecolor()
{
	XSetForeground(Dis, Gc, Px[CurrentColor]) ;
	}
