c -------------------------------------------------------------------- c
c -------------------------------------------------------------------- c
      subroutine simplx(xinit, n, funct, dist, eps, fr)
c -------------------------------------------------------------------- c
c -- parameter: ------------------------------------------------------ c
c ---- I/O xinit:      vector of initial and final x ----------------- c
c ---- I   n:          dimension of the vector x --------------------- c
c ---- I   funct:      minimize function ----------------------------- c
c ---- I   dist:       size of first simplex ------------------------- c
c ---- I   eps:        epsilon --------------------------------------- c
c ---- O   fr:         final value ----------------------------------- c
c -------------------------------------------------------------------- c
c -- This subroutine call next subroutine and function. -------------  c
c --------- first, minmax, newsim, center, reduce, update, epslon ---- c
c -------------------------------------------------------------------- c
      implicit  real*8 (a-h, o-z)
      parameter (maxH=3, maxH5=maxH+5)
c
      dimension xinit(maxH), f(maxH5), x(maxH5, maxH)
      data      ALPHA/-1.0d0/, BETA/0.5d0/, GAMMA/2.0d0/
      external  funct
c -------------------------------------------------------------------- c
      ix0 = n + 2
      ixr = n + 3
      ixe = n + 4
      ixc = n + 5
      call first(n, f, x, funct, xinit, dist)
      if (dist .eq. 0.0d0) return
c
c     open(2, file='simplex.line')
      iter = 0
c                                                ****   main loop   ****
1     continue
c     write(2, '(1hm, 2f10.3)') x(1,1), x(1,2)
c     write(2, '(1hl, 2f10.3)') x(2,1), x(2,2)
c     write(2, '(1hl, 2f10.3)') x(3,1), x(3,2)
c     write(2, '(1hl, 2f10.3)') x(1,1), x(1,2)
c                                           ****   min max search   ****
      call minmax(n, f, ixh, ixs, ixl)
c                                            ****   epsilon check   ****
      eps1 = epslon(n, f)
      write(6, '(31h ####  iteration, f, epsilon = , i5, 2D16.8)')
     &          iter, f(ixl), eps1
      if (iter .gt. 2 .and. eps1 .lt. eps) go to 2
c                                               ****   center set   ****
      call center(n, x, ixh, ix0)
c                                               ****   reflection   ****
      call newsim(n, f, x, funct, ix0, ixh, ALPHA, ixr)
      if (f(ixr) .le. f(ixs)) then
        if (f(ixr) .lt. f(ixl)) then
c                                                ****   expansion   ****
          call newsim(n, f, x, funct, ix0, ixr, GAMMA, ixe)
          if (f(ixe) .lt. f(ixr)) then
            call update(n, f, x, ixe, ixh)
          else
            call update(n, f, x, ixr, ixh)
            end if
        else
          call update(n, f, x, ixr, ixh)
          end if
      else
        if (f(ixr) .lt. f(ixh)) call update(n, f, x, ixr, ixh)
c                                              ****   contraction   ****
        call newsim(n, f, x, funct, ix0, ixh, BETA, ixc)
        if (f(ixc) .lt. f(ixh)) then
          call update(n, f, x, ixc, ixh)
        else
c                                                   ****   reduce   ****
          call reduce(n, f, x, funct, ixl)
          end if
        end if
      iter = iter + 1
      if (iter .gt. 100) go to 2
      go to 1
2     continue
c     close(2)
c
      do 10 j = 1, n
        xinit(j) = x(ixl, j)
10      continue
      fr = f(ixl)
c
      return
      end
c -------------------------------------------------------------------- c
c -------------------------------------------------------------------- c
      subroutine minmax(n, f, ixh, ixs, ixl)
c
      implicit  real*8 (a-h, o-z)
      parameter (maxH=3, maxH5=maxH+5)
      dimension f(maxH5)
c -------------------------------------------------------------------- c
      if (f(1) .gt. f(2)) then
        fmax1 = f(1)
        ixh = 1
        fmax2 = f(2)
        ixs = 2
        fmin1 = f(2)
        ixl = 2
      else
        fmax1 = f(2)
        ixh = 2
        fmax2 = f(1)
        ixs = 1
        fmin1 = f(1)
        ixl = 1
        end if
c
      do 10 i = 3, n+1
        if (f(i) .gt. fmax1) then
          fmax2 = fmax1
          ixs = ixh
          fmax1 = f(i)
          ixh = i
        else if (f(i) .gt. fmax2) then
          fmax2 = f(i)
          ixs = i
          end if
        if (f(i) .lt. fmin1) then
          fmin1 = f(i)
          ixl = i
          end if
10      continue
c
      return
      end
c -------------------------------------------------------------------- c
c -------------------------------------------------------------------- c
      subroutine first(n, f, x, funct, xinit, dist)
c
      implicit  real*8 (a-h, o-z)
      parameter (maxH=3, maxH5=maxH+5)
      dimension f(maxH5), x(maxH5, maxH), xinit(maxH), xx(maxH)
      external  funct
c -------------------------------------------------------------------- c
      do 10 i = 1, n + 1
        do 20 j = 1, n
          x(i, j) = xinit(j)
          xx(j) = x(i, j)
20        continue
        if (i .ne. 1) then
          if (dist .eq. 0.0d0) return
          x(i, i-1) = x(i, i-1) + dist
          xx(i-1) = x(i,i-1)
          end if
        call funct(n, xx, f(i))
10      continue
c
      return
      end
c -------------------------------------------------------------------- c
c -------------------------------------------------------------------- c
      subroutine center(n, x, ixh, ix0)
c
      implicit  real*8 (a-h, o-z)
      parameter (maxH=3, maxH5=maxH+5)
      dimension x(maxH5, maxH)
c -------------------------------------------------------------------- c
      do 10 j = 1, n
        x(ix0, j) = 0.0d0
        do 20 i = 1, n+1
          if (i .ne. ixh) x(ix0, j) = x(ix0, j) + x(i, j)
20        continue
        x(ix0, j) = x(ix0, j) / dfloat(n)
10      continue
c
      return
      end
c -------------------------------------------------------------------- c
c -------------------------------------------------------------------- c
      function   epslon(n, f)
c
      implicit  real*8 (a-h, o-z)
      parameter (maxH=3, maxH5=maxH+5)
      dimension f(maxH5)
c -------------------------------------------------------------------- c
      emean = 0.0d0
      do 10 i = 1, n+1
        emean = emean + f(i)
10      continue
      emean = emean / dfloat(n+1)
      epslon = 0.0d0
      do 20 i = 1, n+1
        epslon = epslon + (f(i) - emean) ** 2
20      continue
      epslon = dsqrt(epslon) / dfloat(n+1)
c
      return
      end
c -------------------------------------------------------------------- c
c -------------------------------------------------------------------- c
      subroutine newsim(n, f, x, funct, ix0, in, prm, iout)
c
      implicit  real*8 (a-h, o-z)
      parameter (maxH=3, maxH5=maxH+5)
      dimension f(maxH5), x(maxH5,maxH), xx(maxH)
      external  funct
c -------------------------------------------------------------------- c
      do 10 j = 1, n
        x(iout, j) = prm * x(in, j) + (1.0d0 - prm) * x(ix0, j)
        xx(j) = x(iout,j)
10      continue
      call funct(n, xx, f(iout))
c
      return
      end
c -------------------------------------------------------------------- c
c -------------------------------------------------------------------- c
      subroutine update(n, f, x, in, iout)
c
      implicit  real*8 (a-h, o-z)
      parameter (maxH=3, maxH5=maxH+5)
      dimension f(maxH5), x(maxH5, maxH)
c -------------------------------------------------------------------- c
      do 10 j = 1 , n
        x(iout, j) = x(in, j)
10      continue
      f(iout) = f(in)
c
      return
      end
c -------------------------------------------------------------------- c
c -------------------------------------------------------------------- c
      subroutine reduce(n, f, x, funct, ixl)
c
      implicit  real*8 (a-h, o-z)
      parameter (maxH=3, maxH5=maxH+5)
      dimension f(maxH5), x(maxH5, maxH), xx(maxH)
      external  funct
c -------------------------------------------------------------------- c
      do 10 i = 1, n+1
        if (i .ne. ixl) then
          do 20 j = 1, n
            x(i, j) = (x(i, j) + x(ixl, j)) / 2.0d0
            xx(j) = x(i, j)
20          continue
          call funct(n, xx, f(i))
          end if
10      continue
c
      return
      end
