/*
*  Program name:             birdC  ver. 2.0
*
*  Program function:         Color-pattern bird-eye view plot
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            August 8, 1989
*
*  Date of revision:         June 9, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -f  JKM file name
*                            -cf Color JKM file name
*                            -lf Legend table file name (JK-LEGEND)
*                            -rz z rad (default 10.0 deg)
*                            -rx x rad (default 20.0 deg)
*                            -in interval (default 1)
*                            -sx x size (default 19.0 cm)
*                            -sy y size (default 27.0 cm)
*                            -sz hscale (default 1.0)
*                            -sc Scale (default calculated)
*                            -xs Xshift   ( default: 0.0 )
*                            -ys Yshift   ( default: 0.0 )
*                            -ci Contour interval (default # 1.0)
*                            -cs Contour start (default # gmin)
*                            -ce Contour end (default # gmax)
*                            -pl square poly mode
*                            -pm: mesh pen number
*                            -pc: contour pen number
*                            -ex: Eye_x position
*                            -ey: Eye_y position
*                            -py: projection screen position
*                            -frame: frame plot
*
*  Input:                    JKM file (program open)
*
*  Output:                   JKG file for the Bird's eye view
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#define	MaxP	10
#define	MaxC	1025
#define	POLY	0
#define	CONTOUR	1

#include "bird_inc.c"

int	Ntable, Color[MaxC], Pattern[MaxC], PatternC[MaxC], Pen_o ;
float	*C ;
float	Xo, Yo, Cstr, Cend, Cint ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, n, inter, stat_frame, contour_poly ;
	char	dsn_h[80], dsn_c[80], dsn_t[80] ;
	void	parm_init(), bird_init(), cont4d(), grid_read(), table_read() ;
/* -------------------------------------------------------------------------- */
	inter = 1 ;
	stat_frame = 0 ;
	contour_poly = CONTOUR ;
	Cstr = -9999.0 ;
	Cend = -9999.0 ;
	Cint = 1.0 ;
	Pen_c = 3 ;
	dsn_h[0] = '\0' ;
	dsn_c[0] = '\0' ;
	dsn_t[0] = '\0' ;
	parm_init(argc, argv) ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-f")  == 0) strcpy(dsn_h, argv[++i]) ;
	  else if (strcmp(argv[i], "-cf") == 0) strcpy(dsn_c, argv[++i]) ;
	  else if (strcmp(argv[i], "-in") == 0) inter = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-pc") == 0) Pen_c = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-frame") == 0) stat_frame = 1 ;
	  else if (strcmp(argv[i], "-cs") == 0) Cstr  = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-ce") == 0) Cend  = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-ci") == 0) Cint  = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-lf") == 0) strcpy(dsn_t, argv[++i]) ;
	  else if (strcmp(argv[i], "-pl") == 0) contour_poly = POLY ;
	  else if (strcmp(argv[i], "-h")  == 0) {
	    fprintf(stderr, " USAGE: birdC -f $ -cf $ -rx # -rz # -in # ") ;
	    fprintf(stderr, "-sx # -sy # -sz # -sc # -xs # -ys # -pm # ") ;
	    fprintf(stderr, "-pc # -ex # -ey # -py # -frame -cs # -ce # ") ;
	    fprintf(stderr, "-ci # -lf $ -pl\n") ;
	    return(0) ;
	    }
	  }
	if (Pen_m == 0 && Pen_c == 0) {
	  fprintf(stderr, " Pen number error.\n") ;
	  return(-1) ;
	  }
	if (Cstr >= Cend) {
	  fprintf(stderr, " Cstr/Cend error.\n") ;
	  return(-1) ;
	  }
/* ----------------------------------------------------------- grid read ---- */
	if (dsn_c[0] != '\0') grid_read(dsn_c, inter, 1) ;
	grid_read(dsn_h, inter, 2) ;
/* ----------------------------------------------------------- bird init ---- */
	bird_init(stat_frame) ;
/* -------------------------------------------------------------------------- */
	if (dsn_c[0] == '\0') {
	  if ((C = (float *)calloc(Nx*Ny, sizeof(*C)))==NULL) {
	    fprintf(stderr, " Can't alloc color grid.\n") ;
	    return(-1) ;
	    }
	  for (n = 0; n < Nx*Ny; n++) C[n] = H[n] ;
	  Cstr = Cstr * Hscale ;
	  Cend = Cend * Hscale ;
	  Cint = Cint * Hscale ;
	  Undc = Und ;
	  }
/* -------------------------------------------------------------------------- */
	if (contour_poly == CONTOUR) {
	  for (n = 0; n < Nx*Ny; n++) {
	    if (H[n] != Und && C[n] != Undc) C[n] = (C[n] - Cstr) / Cint ;
	    else C[n] = -999.0 ;
	    }
	  Cmax = (Cend - Cstr) / Cint ;
	  fprintf(stderr, " Cstr, Cend, Cint, Cmax = %f %f %f %d\n",
	  	Cstr, Cend, Cint, Cmax) ;
	  }
	else Cmax = MaxC - 2 ;
	table_read(dsn_t) ;
/* -------------------------------------------------------------------------- */
	Cpen[0] = 'l' ;
	Cpen[1] = 'l' ;
	if (Pen_m == 0) Cpen[0] = 'm' ;
	if (Pen_c == 0) Cpen[1] = 'm' ;
/* -------------------------------------------------------------------------- */
	cont4d(Nx-2, Nx_c,  -1, Ny-2, Ny_c,  -1, contour_poly) ;
	cont4d(0,    Nx_c-1, 1, Ny-2, Ny_c,  -1, contour_poly) ;
	cont4d(0,    Nx_c-1, 1,    0, Ny_c-1, 1, contour_poly) ;
	cont4d(Nx-2, Nx_c,  -1,    0, Ny_c-1, 1, contour_poly) ;

	free(H) ;
	free(C) ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	cont4d(ixs, ixe, ixd, iys, iye, iyd, contour_poly)
int	ixs, ixe, ixd, iys, iye, iyd, contour_poly ;
{
	int	ix, iy ;
	float	u[5], o[5] ;
	void	cont4d_sub() ;

	if (ixd > 0 && ixe < ixs) return ;
	if (ixd < 0 && ixe > ixs) return ;
	if (iyd > 0 && iye < iys) return ;
	if (iyd < 0 && iye > iys) return ;
	Pen_o = 0 ;
	if (Pen_m != 0) {
	  printf("Lw %d\n", Pen_m) ;
	  Pen_o = Pen_m ;
	  }
	else if (Pen_c != 0) {
	  printf("Lw %d\n", Pen_c) ;
	  Pen_o = Pen_c ;
	  }
	if (ixs < 0)    ixs = 0 ;
	if (iys < 0)    iys = 0 ;
	if (ixs > Nx-2) ixs = Nx-2 ;
	if (iys > Ny-2) iys = Ny-2 ;
	if (ixe < 0)    ixe = 0 ;
	if (iye < 0)    iye = 0 ;
	if (ixe > Nx-2) ixe = Nx-2 ;
	if (iye > Ny-2) iye = Ny-2 ;
	fprintf(stderr, " x = %d to %d by %d, y = %d to %d by %d\n",
	  ixs, ixe, ixd, iys, iye, iyd) ;
	for (ix = ixs; ix != ixe + ixd; ix = ix + ixd) {
	  for (iy = iys; iy != iye + iyd; iy = iy + iyd) {
	    o[0] = C[ix*Ny+iy] ;
	    o[1] = C[ix*Ny+iy+1] ;
	    o[2] = C[(ix+1)*Ny+iy+1] ;
	    o[3] = C[(ix+1)*Ny+iy] ;
	    if (o[0] < 0 || o[1] < 0 || o[2] < 0 || o[3] < 0) continue ;
	    Xo = (float)ix ;
	    Yo = (float)iy ;
	    u[0] = H[ix*Ny+iy] ;
	    u[1] = H[ix*Ny+iy+1] ;
	    u[2] = H[(ix+1)*Ny+iy+1] ;
	    u[3] = H[(ix+1)*Ny+iy] ;
	    cont4d_sub(u, o, contour_poly) ;
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	cont4d_sub(u, o, contour_poly)
float	u[5], o[5] ;
int	contour_poly ;
{
	float	x[MaxP], y[MaxP], e[MaxP], rs, re, rw, ors, ore, where() ;
	float	omin, omax ;
	static	float	xc[5] = {0.0, 0.0, 1.0, 1.0, 0.0} ;
	static	float	yc[5] = {0.0, 1.0, 1.0, 0.0, 0.0} ;
	int	p[MaxP] ;
	int	square, io[5], iomin, iomax ;
	int	i, j, ip, cs, ce, imode, gauss() ; 
	void	add(), data_out() ;

	omin =  99999.0 ;
	omax = -99999.0 ;
	ip = 0 ;
	square = POLY ;
	for (i = 0; i < 4; i++) {
	  if (omin > o[i]) omin = o[i] ;
	  if (omax < o[i]) omax = o[i] ;
	  io[i] = gauss(o[i]) ;
	  if (o[i] == (float)io[i]) o[i] = o[i] + 0.001 ;
	  if (o[i] < 0.0) io[i] = - 1 ;
	  if (io[i] > Cmax) io[i] = Cmax ;
	  if (io[0] != io[i]) square = CONTOUR ;
	  }
	u[4] = u[0] ;
	o[4] = o[0] ;
							/* square */
	if (contour_poly == POLY || square == POLY) {
	  for (i = 0; i < 5; i++) p[i] = -1 ;
	  data_out(io[0], 4, xc, yc, u, p) ;
	  return ;
	  }
							/* polygon */
	iomin = gauss(omin) ;
	if (iomin < -1) iomin = -1 ;
	iomax = gauss(omax) ;
	if (iomax > Cmax) iomax = Cmax ;
	for (i = iomin; i <= iomax; i++) {
	  imode = 0 ;
	       if (io[0]<=i && io[1]> i && io[2]<=i && io[3]> i) imode = 1 ;
	  else if (io[0]> i && io[1]<=i && io[2]> i && io[3]<=i) imode = 2 ;
	  for (j = 0; j < 2; j++) {
	    ors = (float)i     ;
	    ore = (float)(i+1) ;
	    ip = 0 ;
	    for (cs = 0; cs < 4 ; cs++) {
	      if (imode == 1 && j == 0 && (cs == 1 || cs == 2)) continue ;
	      if (imode == 1 && j == 1 && (cs == 0 || cs == 3)) continue ;
	      if (imode == 2 && j == 0 && (cs == 0 || cs == 1)) continue ;
	      if (imode == 2 && j == 1 && (cs == 2 || cs == 3)) continue ;
	      if (o[cs] > ors && o[cs] < ore) {
	        x[ip] = xc[cs] ;
	        y[ip] = yc[cs] ;
	        e[ip] = u[cs] ;
	        p[ip] = -1 ;
	        ip++ ;
	        }
	      ce = cs + 1 ;
	      if (ce == 4) ce = 0 ;
	      rs = where(ors, o[cs], o[ce]) ;
	      re = where(ore, o[cs], o[ce]) ;
	      if (rs < 0.0) {
	        if (re >= 0.0) {
	          add(x, y, e, ip, cs, u[cs], u[ce], re) ;
	          p[ip] = i + 1 ;
	          ip++ ;
	          }
	        }
	      else if (re < 0.0) {
	        add(x, y, e, ip, cs, u[cs], u[ce], rs) ;
	        p[ip] = i ;
	        ip++ ;
	        }
	      else {
	        p[ip]   = i ;
	        p[ip+1] = i + 1 ;
	        if (re < rs) {
	          rw = rs ;
	          rs = re ;
	          re = rw ;
	          p[ip]   = i + 1 ;
	          p[ip+1] = i ;
	          }
	        add(x, y, e, ip, cs, u[cs], u[ce], rs) ;
	        ip++ ;
	        add(x, y, e, ip, cs, u[cs], u[ce], re) ;
	        ip++ ;
	        }
	      }
	    x[ip] = x[0] ;
	    y[ip] = y[0] ;
	    e[ip] = e[0] ;
	    p[ip] = p[0] ;
	    data_out(i, ip, x, y, e, p) ;
	    if (imode == 0) break ;
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	add(x, y, e, ip, cs, ucs, uce, r)
float	x[], y[], e[], ucs, uce, r ;
int	ip, cs ;
{
	if (cs == 0) {
	  x[ip] = 0.0 ;
	  y[ip] = r ;
	  }
	else if (cs == 1) {
	  x[ip] = r ;
	  y[ip] = 1.0 ;
	  }
	else if (cs == 2) {
	  x[ip] = 1.0 ;
	  y[ip] = 1.0 - r ;
	  }
	else {
	  x[ip] = 1.0 - r ;
	  y[ip] = 0.0 ;
	  }
	e[ip] = (uce - ucs) * r + ucs ;
	}
/* -------------------------------------------------------------------------- */
float	where(or, os, oe)
float	or, os, oe ;
{
	if ((or>os && or<oe) || (or>oe && or<os)) return((or-os) / (oe-os)) ;
	else return(-9999.0) ;
	}
/* -------------------------------------------------------------------------- */
void	data_out(it, ip, x, y, e, p)
float	x[], y[], e[] ;
int	it, ip, p[] ;
{

	int	i ;
	float	xd, yd, xdo, ydo ;
	float	xw, yw ;
	void	conv() ;

	if (ip < 3) return ;
	if (Pattern[it] == 0 && Color[it] == 0) return ;
	printf("Fb %d\n", Color  [it]) ;
	printf("Fp %d\n", Pattern[it]) ;
	if (Pattern[it] != 0) printf("Fc %d\n", PatternC[it]) ;
	printf("n\n") ;
	for (i = 0; i <= ip; i++) {
	  xw = (x[i] + Xo) * Dx ;
	  yw = (y[i] + Yo) * Dy ;
	  conv(xw, yw, e[i], &xd, &yd) ;
	  if (i == 0) printf("m %f %f\n", xd, yd) ;
	  else {
	    if (p[i] > -1 && p[i] == p[i-1]) {
	      if (Cpen[1] == 'l' && Pen_c != Pen_o) {
	        printf("Lw %d\n", Pen_c) ;
	        printf("%c %f %f\n", 'm', xdo, ydo) ;
	        Pen_o = Pen_c ;
	        }
	      printf("%c %f %f\n", Cpen[1], xd, yd) ;
	      if (Cpen[1] == 'l' && Pen_m != Pen_o) {
	        printf("Lw %d\n", Pen_m) ;
	        if (i != ip) printf("%c %f %f\n", 'm', xd, yd) ;
	        Pen_o = Pen_m ;
	        }
	      }
	    else printf("%c %f %f\n", Cpen[0], xd, yd);
	    }
	  xdo = xd ;
	  ydo = yd ;
	  }
	printf("f\n") ;
	}
/* -------------------------------------------------------------------------- */
void	grid_read(dsn, inter, ih)
char	dsn[] ;
int	inter, ih ;
{
	int	ix, iy, jx, jy, n, ky ;
	float	hw, uw ;
	FILE	*fp ;

	if (dsn[0] == '\0') fp = stdin ;
	else if ((fp = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open grid data '%s'.\n", dsn) ;
	  exit(-1) ;
	  }
	while ((getc(fp)) != '\n') ;
	fscanf(fp, "%d %d %f %f %f %f %f %f %f",
	  &Nx, &Ny, &Xstr, &Ystr, &Dx, &Dy, &uw, &Hmin, &Hmax) ;
	if (ih == 1) Undc = uw ;
	else Und = uw ;
	n = ((Nx+inter-2)/inter+1) * ((Ny+inter-2)/inter+1) ;
	if (ih == 1 && (C = (float *)calloc(n, sizeof(*C))) == NULL) {
	  fprintf(stderr, " Can't alloc color grid.\n") ;
	  exit(-1) ;
	  }
	else if ((H = (float *)calloc(n, sizeof(*H))) == NULL) {
	  fprintf(stderr, " Can't alloc height grid.\n") ;
	  exit(-1) ;
	  }
	Dx = Dx * inter ;
	Dy = Dy * inter ;
	ky = (Ny + inter - 2) / inter + 1 ;
	for (ix = 0; ix < Nx; ix++) {
	  jx = (ix + inter - 1) / inter ;
	  for (iy = 0; iy < Ny; iy++) {
	    fscanf(fp, "%f", &hw) ;
	    jy = (iy + inter - 1) / inter ;
	    if (ih == 1) C[jx*ky+jy] = hw ;
	    else H[jx*ky+jy] = hw ;
	    }
	  }
	fclose(fp) ;
	Nx = jx + 1 ;
	Ny = jy + 1 ;
	fprintf(stderr, " Grid file read end.\n") ;
	}
/* -------------------------------------------------------------------------- */
void	table_read(dsn_t)
char	dsn_t[] ;
{
	int	i, Colorw, Patternw, Patternc ;
	FILE	*fp ;

	if (dsn_t[0] == '\0') {
	  for (i = 0; i < MaxC; i++) {
/*	    Color[i] = i + 1 ;
	    Pattern[i] = 0 ; */
	    Color[i] = 0 ;
	    Pattern[i] = -2 ;
	    PatternC[i] = 1 ;
	    }
	  Ntable = MaxC ;
	  }
	else {
	  if ((fp = fopen(dsn_t, "r")) == NULL) {
	    fprintf(stderr, " Can't open legend file '%s'.\n", dsn_t) ;
	    exit(-1) ;
	    }
	  Ntable = 0 ;
	  while ((fscanf(fp, "%d %d %d %d", &i, &Colorw, &Patternw, &Patternc)) == 4) {
	    Color[Ntable] = Colorw ;
	    Pattern[Ntable] = Patternw ;
	    PatternC[Ntable] = Patternc ;
	    Ntable++ ;
	    if (Ntable > MaxC) {
	      fprintf(stderr, " Color table Ntable > MaxC.") ;
	      exit(-1) ;
	      }
	    }
	  fclose(fp) ;
	  }
	fprintf(stderr, " Ntable = %d.\n", Ntable) ;
	}
/* -------------------------------------------------------------------------- */
int	gauss(x)
float	x ;
{
	if (x < 0.0) return((int)x-1) ;
	return((int)x) ;
	}
