/*
*  Program name:             contourC  ver. 2.1
*
*  Program function:         Color-Pattern Contour Program
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            August 8, 1989
*
*  Date of revision:         May 23, 1991 Y. Murata
*                            June 9, 1995 for JKG by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*                            July 3, 2003 Add inequality interval option Y. M
*
*  Arguments:                -f:  JKM file name            (JKM File)
*                            -i:  Contour interval         (default = 1.0)
*                            -s:  Contour start val.       (default = min)
*                            -e:  Contour end   val.       (default = max)
*                            -lf: Color/Pattern Table file (JK-MOD File)
*                            -ls: Legend table start pos.  (default = 1)
*                            -bw: Legend box width         (default = 1.0 cm)
*                            -bx: Legend box x-position    (default = Xmax)
*                            -by: Legend box y-position    (default = Ymin)
*                            -pm: mesh line color          (default = 0)
*                            -pl: polygon plot             (default = 0)
*                            -ps: postscript print         (default = 0)
*                            -pc: contour line color       (default = 1)
*                            -head: Header no output       (default = out)
*                            -in: Grid read interval       (default = 1)
*                            -jo: Join same value cell     (default = disable)
*
*  Input:                    JKM file (program open)
*
*  Output:                   JKG file (to stdout)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#define	MAXP	10
#define	MAXC	1025
#define	POLY	0
#define	CONTOUR	1
#define	POST	2

double	Xmin, Ymin, Xmax, Ymax, Xwid, Ywid, Xstr, Ystr, Boxx, Boxy ;
float	Value[MAXC] ;
int	Color[MAXC], Pattern[MAXC], PatternC[MAXC] ;
float	*H ;
float	Und, Hmin, Hmax, Hscale, Hstr, Hend, Hint ;
int	Nx, Ny, Cmax, Cmod ;
int	Pen_m, Pen_c, Join ;
char	Cpen[2] ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	float	boxw, vnormal() ;
	int	i, n, inter, lstr, head, contour_poly ;
	char    dsn_c[80], dsn_t[80] ;
	void	Grid_Read(), Table_Read(), Cont4p(), Cont4s(), Cont4d() ;
/* -------------------------------------------------------------------------- */
	inter  = 1 ;
	Hstr   = -9999.0 ;
	Hend   = -9999.0 ;
	Hint   = 1.0 ;
	Hscale = 1.0 ;
	contour_poly = CONTOUR ;
	Pen_m = 0 ;
	Pen_c = 1 ;
	lstr = 1 ;
	boxw = 1.0 ;
	head = 1 ;
	Boxx = 9.0e-9 ;
	Boxy = 9.0e-9 ;
	dsn_c[0] = '\0' ;
	dsn_t[0] = '\0' ;
	Cmod = 0 ;
	Cmax = 0 ;
	Join = 0 ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-f")  == 0) strcpy(dsn_c, argv[++i]) ;
	  else if (strcmp(argv[i], "-in") == 0) inter = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-s")  == 0) Hstr = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-e")  == 0) Hend = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-i")  == 0) Hint = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-lf") == 0) strcpy(dsn_t, argv[++i]) ;
	  else if (strcmp(argv[i], "-ls") == 0) lstr = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-bw") == 0) boxw = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-bx") == 0) Boxx = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-by") == 0) Boxy = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-pl") == 0) contour_poly = POLY    ;
	  else if (strcmp(argv[i], "-ps") == 0) contour_poly = POST    ;
	  else if (strcmp(argv[i], "-pm") == 0) Pen_m = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-pc") == 0) Pen_c = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-head") == 0) head = 0 ;
	  else if (strcmp(argv[i], "-jo") == 0) Join = 1 ;
	  else if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr," USAGE: contourC -f $ -in # -s # -e # -i # ") ;
	    fprintf(stderr,"-lf $ -ls # -bw # -bx # -by # -pl -ps -pm # -pc") ;
	    fprintf(stderr," # -head\n") ;
	    return(0) ;
	    }
	  }
	if (contour_poly == POST) boxw = 0.0 ;
	Grid_Read(dsn_c, inter) ;

	if (head == 1) {
/*	if (head == 1 && contour_poly != POST) { */
	  printf("A %lf %lf %lf %lf\n", Xmin, Ymin, Xmax, Ymax) ;
	  printf("Lw 5\n") ;
	  printf("m %lf %lf\n", Xmin, Ymin) ;
	  printf("l %lf %lf\n", Xmin, Ymax) ;
	  printf("l %lf %lf\n", Xmax, Ymax) ;
	  printf("l %lf %lf\n", Xmax, Ymin) ;
	  printf("l %lf %lf\n", Xmin, Ymin) ;
	  }

	if (Boxx == 9.0e-9) Boxx = Xmax ;
	if (Boxy == 9.0e-9) Boxy = Ymin ;
	if (Hstr != Hend) {
	  Cmax = (Hend - Hstr) / Hint + 0.5 ;
	  for (n = 0; n < Nx * Ny; n++) {
	    if (H[n] != Und) {
	      H[n] = (H[n] - Hstr) / Hint ;
/*	      if (H[n] <     0) H[n] = 0 ;
	      if (H[n] >= Cmax) H[n] = Cmax - 1 ; */
	      }
	    else H[n] = -999.0 ;
	    }
	  if (dsn_t[0] != '\0') Table_Read(lstr, boxw, dsn_t) ;
	  fprintf(stderr, " Hstr, Hend, Hint, Cmax = %f %f %f %d %d\n",
	    Hstr, Hend, Hint, Cmax, MAXC-2) ;
	  }
	else {
	  if (dsn_t[0] != '\0') Table_Read(lstr, boxw, dsn_t) ;
	  for (n = 0; n < Nx * Ny; n++) {
	    if (H[n] != Und) H[n] = vnormal(H[n]) ;
	    else H[n] = -999.0 ;
	    }
	  Hstr = 1.0 ;
	  Hend = (float)Cmax ;
	  Hint = 1.0 ;
	  }
	Und = -999.0 ;
	if (Cmax > MAXC-2) {
	  fprintf(stderr, " Cmax > MAXC-2.\n") ;
	  return(-1) ;
	  }
	Cpen[0] = 'm' ;
	Cpen[1] = 'l' ;
	if (Pen_m != 0) Cpen[0] = 'l' ;
	if (Pen_c == 0) Cpen[1] = 'm' ;

	if (Pen_c != 0 && contour_poly != POST) printf("Lw %d\n", Pen_c) ;
             if (contour_poly == POLY) Cont4p() ;
        else if (contour_poly == POST) Cont4s() ;
	else                           Cont4d() ;
	free(H) ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
float	vnormal(h)
float	h ;
{
	int	i ;
	float	v ;

	if (h < Value[0])
	  v = (h - Hmin) / (Value[0] - Hmin) ;
	else if (h > Value[Cmax-1])
	  v = (Hmax - h) / (Hmax - Value[Cmax-1]) + (float)Cmax ;
	else {
	  for (i = 1; i < Cmax; i++) {
	    if (h < Value[i]) {
	      v = (h - Value[i-1]) / (Value[i] - Value[i-1]) + (float)i ;
	      break ;
	      }
	    }
	  }
	return(v) ;
	}
/* -------------------------------------------------------------------------- */
float	vreverse(h)
float	h ;
{
	int	i ;
	float	v ;

	i = (int)h ;
	if (i == 0)
	  v = h * (Value[0] - Hmin) + Hmin ;
	else if (i == Cmax)
	  v = (h - (float)i) * (Hmax - Value[i-1]) + Value[i-1] ;
	else
	  v = (h - (float)i) * (Value[i] - Value[i-1]) + Value[i-1] ;
	return(v) ;
	}
/* -------------------------------------------------------------------------- */
void	conv(x, y, xd, yd)
float	x, y ;
double	*xd, *yd ;
{
	*xd = x + Xmin ;
	*yd = y + Ymin ;
	}
/* -------------------------------------------------------------------------- */
void	Cont4s()
{
	int	ix, iy, io, gauss() ;
	double	x, y ;

	for (ix = 0; ix < Nx-1; ix++) {
	  x = Xmin + (double)ix * Xwid ;
	  for (iy = 0; iy < Ny-1; iy++) {
	    if ( H[ix*Ny+iy] == Und ) continue ;
	    y = Ymin + (double)iy * Ywid ;
	    io = gauss(H[ix*Ny+iy]) ;
	    if (io >= 0 && io < Cmax) {
	      printf("m %f %f\n", x, y) ;
	      printf("%%%f P\n", (float)io/100.0) ;
	      }
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	Cont4p()
{
	int	ix, iy, io, gauss() ;
	static	int	p[5]  = { -1,  -1,  -1,  -1,  -1} ;
	static	float	xs[5] = {0.0, 1.0, 1.0, 0.0, 0.0} ;
	static	float	ys[5] = {0.0, 0.0, 1.0, 1.0, 0.0} ;
	void	data_out() ;

	for (ix = 0; ix < Nx-1; ix++) {
	  fprintf(stderr, " Contour-Line = %d / %d\n", ix+1, Nx) ;
	  Xstr = (double)ix ;
	  for (iy = 0; iy < Ny-1; iy++) {
	    if (H[ix*Ny+iy] == Und) continue ;
	    Ystr = (double)iy ;
	    io = gauss(H[ix*Ny+iy]) ;
	    if (io < 0   ) io = 0 ;
	    if (io > Cmax) io = Cmax ;
	    if (io >= 0 && io < Cmax) data_out(io, 4, xs, ys, p) ;
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	Cont4d()
{
	int	ix, iy, io[5], gauss(), cont ;
	static	int	p[5]  = {-1, -1, -1, -1, -1} ;
	static	float	xs[5] = {0.0, 1.0, 1.0, 0.0, 0.0} ;
	static	float	ys[5] = {0.0, 0.0, 1.0, 1.0, 0.0} ;
	float	xc[5], yc[5], o[5], c1, c2, c3, c4 ;
	double	Ystro ;
	void	Cont4d_sub(), data_out() ;

	xc[0] = 0.0 ;
	yc[0] = 0.0 ;
	xc[1] = 0.0 ;
	xc[2] = 1.0 ;
	xc[3] = 1.0 ;
	yc[3] = 0.0 ;
	xc[4] = 0.0 ;
	yc[4] = 0.0 ;
	for (ix = 0; ix < Nx-1; ix++) {
	  fprintf(stderr, " Contour-Line = %d / %d\n", ix+1, Nx) ;
	  Xstr = (double)ix ;
	  cont = 0 ;
	  for (iy = 0; iy < Ny-1; iy++) {
	    c1 = H[ix*Ny+iy] ;
	    c2 = H[ix*Ny+iy+1] ;
	    c3 = H[(ix+1)*Ny+iy+1] ;
	    c4 = H[(ix+1)*Ny+iy] ;
	    if (c1==Und || c2==Und || c3==Und || c4==Und) {
	      if (cont != 0) {
	        cont = 0 ;
	        Ystr = Ystro ;
	        if (io[0] >= 0 && io[0] < Cmax)
	          data_out(io[0], 4, xc, yc, p) ;
	        }
	      continue ;
	      }
	    Ystr = (double)iy ;
	    io[0] = gauss(c1) ;
	    io[1] = gauss(c2) ;
	    io[2] = gauss(c3) ;
	    io[3] = gauss(c4) ;
	    if (io[0]!=io[1] || io[1]!=io[2] || io[2]!=io[3] || Join==0) {
	      if (cont != 0) {
	        cont = 0 ;
	        Ystr = Ystro ;
	        if (io[0] >= 0 && io[0] < Cmax)
	          data_out(io[0], 4, xc, yc, p) ;
	        Ystr = (double)iy ;
	        }
	      o[0] = c1 ;
	      o[1] = c2 ;
	      o[2] = c3 ;
	      o[3] = c4 ;
	      Cont4d_sub(o) ;
	      continue ;
	      }
	    else if (cont == 0) {
	      cont = 1 ;
	      yc[1] = 1.0 ;
	      yc[2] = 1.0 ;
	      Ystro = Ystr ;
	      }
	    else {
	      yc[1] = yc[1] + 1.0 ;
	      yc[2] = yc[2] + 1.0 ;
	      }
	    }
	  if (cont != 0) {
	    Ystr = Ystro ;
	    if (io[0] >= 0 && io[0] < Cmax)
	      data_out(io[0], 4, xc, yc, p) ;
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	Cont4d_sub(o)
float	o[5] ;
{
	float	x[MAXP], y[MAXP], rs, re, rw, ors, ore, where() ;
	float	omin, omax ;
	static	float	xc[5] = {0.0, 0.0, 1.0, 1.0, 0.0} ;
	static	float	yc[5] = {0.0, 1.0, 1.0, 0.0, 0.0} ;
	int	p[MAXP] ;
	int	ip, io[5], iomin, iomax ;
	int	i, j, cs, ce, imode, gauss() ;
	void	data_out(), add() ;

	omin =  99999.0 ;
	omax = -99999.0 ;
	for (i = 0; i < 4; i++) {
	  if (omin > o[i]) omin = o[i] ;
	  if (omax < o[i]) omax = o[i] ;
	  io[i] = gauss(o[i]) ;
	  if (o[i] == (float)io[i]) o[i] = o[i] + 0.001 ;
	  if (io[i] < 0   ) io[i] = -1 ;
	  if (io[i] > Cmax) io[i] = Cmax+1 ;
	  }
	o[4] = o[0] ;
	iomin = gauss(omin) ;
	if (iomin < 0) iomin = 0 ;
	iomax = gauss(omax) ;
	if (iomax > Cmax-1) iomax = Cmax-1 ;
	for (i = iomin; i <= iomax; i++) {
	  imode = 0 ;
	       if (io[0] <= i && io[1] >  i && io[2] <= i && io[3] >  i) imode = 1 ;
	  else if (io[0] >  i && io[1] <= i && io[2] >  i && io[3] <= i) imode = 2 ;
	  for (j = 0; j < 2; j++) {
	    ors = (float)i     ;
	    ore = (float)(i+1) ;
	    ip = 0 ;
	    for (cs = 0; cs < 4 ; cs++) {
	      if (imode == 1 && j == 0 && (cs == 1 || cs == 2)) continue ;
	      if (imode == 1 && j == 1 && (cs == 0 || cs == 3)) continue ;
	      if (imode == 2 && j == 0 && (cs == 0 || cs == 1)) continue ;
	      if (imode == 2 && j == 1 && (cs == 2 || cs == 3)) continue ;
	      if (o[cs] > ors && o[cs] < ore) {
	        x[ip] = xc[cs] ;
	        y[ip] = yc[cs] ;
	        p[ip] = -1 ;
	        ip++ ;
	        }
	      ce = cs + 1 ;
	      if (ce == 4) ce = 0 ;
	      rs = where(ors, o[cs], o[ce]) ;
	      re = where(ore, o[cs], o[ce]) ;
	      if (rs < 0.0) {
	        if (re >= 0.0) {
	          add(x, y, ip, cs, re) ;
	          p[ip] = i + 1 ;
	          ip++ ;
	          }
	        }
	      else if (re < 0.0) {
	        add(x, y, ip, cs, rs) ;
	        p[ip] = i ;
	        ip++ ;
	        }
	      else {
	        p[ip]   = i ;
	        p[ip+1] = i + 1 ;
	        if (re < rs) {
	          rw = rs ;
	          rs = re ;
	          re = rw ;
	          p[ip]   = i + 1 ;
	          p[ip+1] = i ;
	          }
	        add(x, y, ip, cs, rs) ;
	        ip++ ;
	        add(x, y, ip, cs, re) ;
	        ip++ ;
	        }
	      }
	    x[ip] = x[0] ;
	    y[ip] = y[0] ;
	    p[ip] = p[0] ;
	    data_out(i, ip, x, y, p) ;
	    if (imode == 0) break ;
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	add(x, y, ip, cs, r)
float	x[], y[], r ;
int	ip, cs ;
{
	if (cs == 0) {
	  x[ip] = 0.0 ;
	  y[ip] = r ;
	  }
	else if (cs == 1) {
	  x[ip] = r ;
	  y[ip] = 1.0 ;
	  }
	else if (cs == 2) {
	  x[ip] = 1.0 ;
	  y[ip] = 1.0 - r ;
	  }
	else {
	  x[ip] = 1.0 - r ;
	  y[ip] = 0.0 ;
	  }
	}
/* -------------------------------------------------------------------------- */
float	where(or, os, oe)
float	or, os, oe ;
{
	int	vr, vs, ve ;
	float	vreverse() ;

	if (Cmod == 0) {
	  if ((or > os && or < oe) || (or > oe && or < os))
	    return((or - os) / (oe - os)) ;
	  else return(-9999.0) ;
	  }
	else {
	  vs = vreverse(os) ;
	  ve = vreverse(oe) ;
	  vr = Value[(int)(vr+0.5)] ;
	  return((vr - vs) / (ve - vs)) ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	data_out(nt, ip, x, y, p)
float	x[], y[] ;
int	nt, ip, p[] ;
{
	int	i ;
	double	xd, yd ;
	float	xw, yw ;
	void	conv() ;

	if (ip < 3) return ;
	if (Pattern[nt] != 0 || Color[nt] != 0) {
	  printf("Fb %d\n", Color[nt]) ;
	  printf("Fp %d\n", Pattern[nt]) ;
	  if (Pattern[nt] != 0) printf("Fc %d\n", PatternC[nt]) ;
	  }
	else return ;
	printf("n\n") ;
	for ( i = 0; i <= ip; i++ ) {
	  xw = (x[i] + Xstr) * Xwid ;
	  yw = (y[i] + Ystr) * Ywid ;
	  conv(xw, yw, &xd, &yd) ;
	  if (i == 0)
	    printf("m %lf %lf\n", xd, yd) ;
	  else if (p[i] > -1 && p[i] == p[i-1])
	    printf("%c %lf %lf\n", Cpen[1], xd, yd);
	  else
	    printf("%c %lf %lf\n", Cpen[0], xd, yd);
	  }
	printf("f\n") ;
	}
/* -------------------------------------------------------------------------- */
void	Grid_Read(dsn_c, inter)
char	*dsn_c ;
int	inter ;
{
	int	ix, iy, jx, jy, mx, my ;
	float	hw ;
	char	c ;
	FILE	*fp ;

	if (dsn_c[0] == '-' || dsn_c[0] == '\0') {
	  fprintf(stderr, " Read Grid-data from standard input.\n") ;
	  fp = stdin ;
	  }
	else if ((fp = fopen(dsn_c, "r")) == NULL) {
	  fprintf(stderr, " Can't open color grid '%s'.\n", dsn_c) ;
	  exit(-1) ;
	  }
	while (1) {
	  c = getc(fp) ;
	  if (c == '\n') break ;
	  if (c == EOF) {
	    fprintf(stderr, "Can't read %s\n", dsn_c) ;
	    fclose(fp) ;
	    exit(-1) ;
	    }
	  }
	if (fscanf(fp, "%d %d %lf %lf %lf %lf %f %f %f",
	  &Nx, &Ny, &Xmin, &Ymin, &Xwid, &Ywid, &Und, &Hmin, &Hmax) != 9) {
	  fprintf(stderr, "Can't read %s\n", dsn_c) ;
	  fclose(fp) ;
	  exit(-1) ;
	  }
	mx = ((Nx + inter - 2) / inter + 1) ;
	my = ((Ny + inter - 2) / inter + 1) ;
	if ((H = (float *)calloc(mx*my, sizeof(*H))) == NULL) {
	  fprintf(stderr, " Can't alloc color grid.\n") ;
	  exit(-1) ;
	  }
	for (ix = 0; ix < Nx; ix++) {
	  jx = ((ix + inter - 1) / inter) * my ;
	  for (iy = 0; iy < Ny; iy++) {
	    if (fscanf(fp, "%f", &hw) != 1) {
	      fprintf(stderr, "contourC: read error\n") ;
	      exit(-1) ;
	      }
	    jy = (iy + inter - 1) / inter ;
	    H[jx+jy] = hw ;
	    if (Hmin > hw) Hmin = hw ;
	    if (Hmax < hw) Hmax = hw ;
	    }
	  }
	if (fp != stdin) fclose(fp) ;
	Nx = mx ;
	Ny = my ;
	Xwid = Xwid * (double)inter ;
	Ywid = Ywid * (double)inter ;
	Xmax = Xmin + Xwid * (double)(Nx-1) ;
	Ymax = Ymin + Ywid * (double)(Ny-1) ;
	}
/* -------------------------------------------------------------------------- */
void	Table_Read(lstr, boxw, dsn_t)
int	lstr ;
float	boxw ;
char	*dsn_t ;
{
	int	i, ntable, Colorw, Patternw, Patternc ;
	float	w ;
	FILE	*fp ;
	void	Draw_Legend() ;

	if (boxw != 0.0) {
	  printf("Ss %f\n", boxw*0.35/2.54*72.0) ;
	  printf("Sr 0.0\n") ;
	  printf("Sh 0.0\n") ;
	  printf("Sv 0.5\n") ;
	  printf("m %lf %lf\n", Boxx, Boxy) ;
	  }
	if (dsn_t[0] == '\0') for (i = 0; i < MAXC; i++) {
	  Color[i] = 0 ;
	  Pattern[i] = 0 ;
	  PatternC[i] = 0 ;
	  }
	else {
	  if ((fp = fopen(dsn_t, "r")) == NULL) {
	    fprintf(stderr, " Can't open color table file.\n") ;
	    exit(-1) ;
	    }
	  i = 0 ;
	  ntable = 0 ;
	  while ((fscanf(fp, "%f %d %d %d", &w, &Colorw, &Patternw, &Patternc)) == 4) {
	    if (i >= lstr-1) {
	      if (Cmax == 0 || ntable < Cmax) {
	        Value[ntable] = w ;
	        Color[ntable] = Colorw ;
	        Pattern[ntable] = Patternw ;
	        PatternC[ntable] = Patternc ;
	        if (boxw != 0.0) Draw_Legend(boxw,i,ntable,Colorw,Patternw,Patternc,Cmax) ;
	        }
	      ntable++ ;
	      }
	    i++ ;
	    }
	  }
	if (Cmax == 0) Cmax = ntable ;
	fclose(fp) ;
	Color[Cmax] = Color[Cmax-1] ;
	}
/* -------------------------------------------------------------------------- */
void	Draw_Legend(boxw, nc, nt, cl, pt, pc, Cmax)
float	boxw ;
int	nc, nt, cl, pt, pc ;
{
	char	text[80] ;
	float	vs, ve ;

	printf("Lw 2\n") ;
	if (pt != 0 || cl != 0) {
	  printf("Fb %d\n", cl) ;
	  printf("Fp %d\n", pt) ;
	  if (pt != 0) printf("Fc %d\n", pc) ;
	  }
	else return ;
	printf("n\n") ;
	printf("rm %f %f\n", boxw,     boxw*(float)nc) ;
	printf("rl %f %f\n", boxw,     boxw*(float)nc+boxw*0.6) ;
	printf("rl %f %f\n", boxw*2.0, boxw*(float)nc+boxw*0.6) ;
	printf("rl %f %f\n", boxw*2.0, boxw*(float)nc) ;
	printf("rl %f %f\n", boxw,     boxw*(float)nc) ;
	printf("f\n") ;
	if (Hstr != Hend) {
	  vs = Hstr+Hint*(float)nt ;
	  ve = Hstr+Hint*(float)(nt+1) ;
	  if ((int)(Hint*1000.0+0.5)%10 != 0) {
	    if (nt == 0) sprintf(text, "s Bellow %.3f\0", vs) ;
	    else if (nt == Cmax-1) sprintf(text, "Above %.3f\0", ve) ;
	    else sprintf(text, "s %.3f ~ %.3f\0", vs, ve) ;
	    }
	  else if ((int)(Hint*100.0+0.5)%10 != 0) {
	    if (nt == 0) sprintf(text, "s Bellow %.2f\0", vs) ;
	    else if (nt == Cmax-1) sprintf(text, "s Above %.2f\0", ve) ;
	    else sprintf(text, "s %.2f ~ %.2f\0", vs, ve) ;
	    }
	  else {
	    if (nt == 0) sprintf(text, "s Bellow %.1f\0", vs) ;
	    else if (nt == Cmax-1) sprintf(text, "s Above %.1f\0", ve) ;
	    else sprintf(text, "s %.1f ~ %.1f\0", vs, ve) ;
	    }
	  }
	else {
	  if (nt == 0) sprintf(text, "s Bellow %.2f\0", Value[0]) ;
	  else if (nt == Cmax-1) sprintf(text, "Above %.2f\0", Value[Cmax-1]) ;
	  else sprintf(text, "s %.2f ~ %.2f\0", Value[nt-1], Value[nt]) ;
	  }
	printf("rm %f %f\n", boxw*2.2, boxw*(float)nc+boxw*0.3);
	printf("Lw 1\n") ;
	printf("%s\n", text) ;
	}
/* -------------------------------------------------------------------------- */
int	gauss(x)
float	x ;
{
	if (x < 0.0) return((int)x - 1) ;
	return((int)x) ;
	}
