/*
*  Program name:             contourP  ver. 2.0
*
*  Program function:         Color-Pattern Contour Program
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            August 8, 1989
*
*  Date of revision:         May 23, 1991 Y. Murata
*                            June 9, 1995 for JKG by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -f     JKM file name        ( JKM file )
*                            -pm    mesh pen number      ( default = 0 )
*                            -head  Header no output     ( default = out )
*                            -frame Frame no output      ( default = out )
*                            -in    Grid read interval   ( default = 1 )
*
*  Input:                    JKM file (program open)
*
*  Output:                   JKG file (to staout)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

double	Xmin, Ymin, Xmax, Ymax, Xwid, Ywid ;
int	*C ;
int	Und, Cmin, Cmax ;
int	Nx, Ny ;
int	Pen_m ;
char	Cpen ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, inter, head, frame ;
	char    dsn[80] ;
	void	Grid_Read(), Cont4p() ;
/* -------------------------------------------------------------------------- */
	inter  = 1 ;
	Pen_m = 0 ;
	head = 1 ;
	dsn[0] = '\0' ;
	frame = 1 ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-f")  == 0) strcpy(dsn, argv[++i]) ;
	  else if (strcmp(argv[i], "-in") == 0) inter = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-pm") == 0) Pen_m = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-head") == 0) head = 0 ;
	  else if (strcmp(argv[i], "-frame") == 0) frame = 0 ;
	  else if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr, " USAGE: contourP -f $ -in # -pm # -head -frame\n") ;
	    return(0) ;
	    }
	  }
	Grid_Read(dsn, inter) ;
	Cpen = 'm' ;
	if ( Pen_m != 0 ) Cpen = 'l' ;
	if ( head == 1 ) {
	  printf("A %lf %lf %lf %lf\n", Xmin, Ymin, Xmax, Ymax) ;
	  }
	if ( frame == 1 ) {
	  printf("Lw 5\n") ;
	  printf("m %lf %lf\n", Xmin, Ymin) ;
	  printf("l %lf %lf\n", Xmin, Ymax) ;
	  printf("l %lf %lf\n", Xmax, Ymax) ;
	  printf("l %lf %lf\n", Xmax, Ymin) ;
	  printf("l %lf %lf\n", Xmin, Ymin) ;
	  }
	if (Pen_m != 0)  printf("Lw %d\n", Pen_m) ;
        Cont4p() ;
	if (head == 1) printf("W %lf %lf %lf %lf\n", Xmin, Ymin, Xmax, Ymax) ;
	free(C) ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	Cont4p()
{
	int	ix, iy, n ;
	double	x1, y1, x2, y2 ;

	for (ix = 0; ix < Nx-1; ix++) {
		fprintf(stderr, " Contour-Line = %d / %d\n", ix+1, Nx) ;
		x1 = (double)ix * Xwid + Xmin ;
		x2 = x1 + Xwid ;
		for ( iy = 0; iy < Ny-1; iy++ ) {
			n = ix * Ny + iy ;
			if (C[n] == Und || C[n] == 0) continue ;
			y1 = (double)iy * Ywid + Ymin ;
			y2 = y1 + Ywid ;
			printf("Fb %d\n", -C[n]) ;
			printf("n\n") ;
			printf("m %lf %lf\n", x1, y1) ;
			printf("%c %lf %lf\n", Cpen, x1, y2) ;
			printf("%c %lf %lf\n", Cpen, x2, y2) ;
			printf("%c %lf %lf\n", Cpen, x2, y1) ;
			if (Cpen == 'l') printf("l %lf %lf\n", x1, y1) ;
			printf("f\n") ;
			}
		}
	}
/* -------------------------------------------------------------------------- */
void	Grid_Read(dsn, inter)
char	dsn[] ;
int	inter ;
{
	int	ix, iy, jx, jy, mx, my, hw ;
	FILE	*fp ;

	if (dsn[0] == '-' || dsn[0] == '\0') {
		fprintf(stderr, " Read Grid-data from standard input.\n") ;
		fp = stdin ;
		}
	else if ((fp = fopen(dsn, "r")) == NULL) {
		fprintf(stderr, " Can't open grid file '%s'.\n", dsn) ;
		exit(-1) ;
		}
	while ( (ix = getc(fp)) != '\n' ) if (ix == EOF) exit(0) ;
	fscanf(fp, "%d %d %lf %lf %lf %lf %d %d %d",
		&Nx, &Ny, &Xmin, &Ymin, &Xwid, &Ywid, &Und, &Cmin, &Cmax) ;
	mx = ((Nx + inter - 2) / inter + 1) ;
	my = ((Ny + inter - 2) / inter + 1) ;
	if ( (C = (int *)calloc(mx*my, sizeof(*C))) == NULL ) {
		fprintf(stderr, " Can't alloc color grid.\n") ;
		exit(-1) ;
		}
	for (ix = 0; ix < Nx; ix++) {
		jx = ((ix + inter - 1) / inter) * my ;
		for (iy = 0; iy < Ny; iy++) {
			if (fscanf(fp, "%d", &hw) != 1) {
				fprintf(stderr, "contourP: read error\n") ;
				exit(-1) ;
				}
			jy = (iy + inter - 1) / inter ;
			C[jx+jy] = hw ;
			}
		}
	while ( (ix = getc(fp)) != '\n' )  ;
	if (fp != stdin) fclose(fp) ;
	Nx = mx ;
	Ny = my ;
	Xwid = Xwid * (double)inter ;
	Ywid = Ywid * (double)inter ;
	Xmax = Xmin + Xwid * (double)(Nx-1) ;
	Ymax = Ymin + Ywid * (double)(Ny-1) ;
	fprintf(stderr, " File read end.\n") ;
	}
