/*
*  Program name:             g_LH  ver. 2.0
*
*  Program function:         Draw Low & High marks (L/H) of JKM data
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            July 31, 1990
*
*  Date of revision:         May 28, 1991 by Y.Murata
*                            May 26, 1996 rewrite using C by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -size # size of the marks L/M in po.
*                                       (default: 6po)
*                            -r #    search radius. (default: 3)
*                            -m      draw marks in minus area
*                            -p      draw marks in plus area
*
*  Input:                    JKM data (from stdin)
*
*  Output:                   JKG fotmat (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "jkm_lib.c"

#define       D(a, b)         ((a) * Ny + (b))

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	long	i, nrad, nrad2, ix, iy, jx, jy, ip, im, n1, n2, labs(), mp ;
	float	size, x, y ;
	double	atof() ;
	void	JKM_Read() ;

	size = 6.0 ;
	nrad  = 3 ;
	nrad2 = 0 ;
	mp = 0 ;
	for (i = 1; i < argc; i++) {
	  if (strcmp("-h", argv[i]) == 0) {
	    fprintf(stderr, " USAGE: g_LH {-size sm -r radius -r2 radius }\n") ;
	    return(0) ;
	    }
	  else if (strcmp("-size", argv[i]) == 0) size  = atof(argv[++i]) ;
	  else if (strcmp("-r",    argv[i]) == 0) nrad  = atoi(argv[++i]) ;
	  else if (strcmp("-r2",   argv[i]) == 0) nrad2 = atoi(argv[++i]) ;
	  else if (strcmp("-m",    argv[i]) == 0) mp = -1 ;
	  else if (strcmp("-p",    argv[i]) == 0) mp =  1 ;
	  }
	if (size <= 0.0 || nrad <= 0) {
	  fprintf(stderr, " Argument error.\n") ;
	  return(-1) ;
	  }
	if (nrad < nrad2) nrad2 = nrad ;
	JKM_Read(1) ;
	printf("Ss %f\n", size) ;
	printf("Sr 0\n") ;
	printf("Sh 0.5\n") ;
	printf("Sv 0.5\n") ;
	for (ix=nrad; ix<Nx-nrad; ix++) for (iy=nrad; iy<Ny-nrad; iy++) {
	  ip = 0 ;
	  im = 0 ;
	  n1 = D(ix, iy) ;
	  if (G[n1] == Und) continue ;
	  if (mp == -1 && G[n1] > 0.0) continue ;
	  if (mp ==  1 && G[n1] < 0.0) continue ;
	  if (G[n1-Ny] > G[n1] && G[n1] > G[n1+Ny]) continue ;
	  if (G[n1-Ny] < G[n1] && G[n1] < G[n1+Ny]) continue ;
	  if (G[n1-1]  > G[n1] && G[n1] > G[n1+1] ) continue ;
	  if (G[n1-1]  < G[n1] && G[n1] < G[n1+1] ) continue ;
	  for (jx= -nrad; jx<=nrad; jx++) for (jy= -nrad; jy<=nrad; jy++) {
	    if (jx == 0 && jy == 0) continue ;
	    n2 = D(ix+jx, iy+jy) ;
	    if (G[n2]==Und && min(labs(jx),labs(jy))<=nrad/3) {
	      ip = im = 1 ;
	      break ;
	      }
	    if (G[n1] < G[n2]) ip = 1 ;
	    if (G[n1] > G[n2]) im = 1 ;
            if (G[n1] == G[n2] && n1 > n2) {
	      ip = 1 ;
	      im = 1 ;
	      }
	    }
	  if (nrad2 != 0) {
	    if (ip == 0) {
	      for (i = 1; i <= nrad; i++) {
	        if (G[D(ix+i, iy)]>G[D(ix+i-1, iy)]) ip = 1 ;
	        if (G[D(ix-i, iy)]>G[D(ix-i+1, iy)]) ip = 1 ;
	        if (G[D(ix, iy+i)]>G[D(ix, iy+i-1)]) ip = 1 ;
	        if (G[D(ix, iy-i)]>G[D(ix, iy-i+1)]) ip = 1 ;
	        if (ip == 1) break ;
	        }
	      }
	    if (im == 0) {
	      for (i = 1; i <= nrad; i++) {
	        if (G[D(ix+i, iy)]<G[D(ix+i-1, iy)]) im = 1 ;
	        if (G[D(ix-i, iy)]<G[D(ix-i+1, iy)]) im = 1 ;
	        if (G[D(ix, iy+i)]<G[D(ix, iy+i-1)]) im = 1 ;
	        if (G[D(ix, iy-i)]<G[D(ix, iy-i+1)]) im = 1 ;
	        if (im == 1) break ;
	        }
	      }
	    }
	  if (ip == 0 || im == 0) {
	    x = Xstr + (double)ix * Dx ;
	    y = Ystr + (double)iy * Dy ;
	    printf("m %f %f\n", x, y) ;
	    if (ip == 0) printf("s H\n") ;
	    if (im == 0) printf("s L\n") ;
	    }
	  }
	free(G) ;
	return(0) ;
	}
