/*
*  Program name:             g_lattic  ver. 2.0
*
*  Program function:         JKM -> ARC/INFO lattice export file
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 12, 1991
*
*  Date of revision:         June 30, 1995 rewrite using C by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. zscale (default = 1)
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   ARC-INFO lattice export file ascii (to stdout)
*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>

#include "jkm_lib.c"

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ix, iy ;
	long	n ;
	double	atof(), zscale=1.0 ;
	void	JKM_Read() ;

	if (argc > 1) {
	  if (!strcmp(argv[1], "-h")) {
	    fprintf(stderr, " USAGE: g_lattic {zscale}\n") ;
	    return(0) ;
	    }
	  zscale = atof(argv[1]) ;
	  if (zscale == 0.0) {
	    fprintf(stderr, " Argument zscale error.\n") ;
	    return(-1) ;
	    }
	  }

	JKM_Read(1) ;

	for (n = 0; n < Nx*Ny; n++) {
	  if (G[n] == Und) G[n] = -1.0 ;
	  else G[n] = G[n] * zscale ;
	  }

	printf("EXP  0 /JKM_GRID_DATA.E00\n") ;
	printf("LAT  2\n") ;
	printf("%10d%10d\n", Nx, Ny) ;
	printf("%14.7E%14.7E%14.7E%14.7E\n", Xstr, Ystr, Dx, Dy) ;
	printf("%14.7E%14.7E\n", Gmin*zscale, Gmax*zscale) ;
	for (iy = Ny-1; iy > -1; iy--) {
	  n = 0 ;
	  for (ix = 0; ix < Nx; ix++) {
	    n++ ;
	    printf("%14.7E", G[ix*Ny+iy]) ;
	    if (n%4 == 0) printf("\n") ;
	    }
	  if (n%4 != 0) printf("\n") ;
	  }
	printf("%10d%10d%10d%10d%10d%10d%10d\n", -1, 0, 0, 0, 0, 0, 0) ;
	printf("EOS\n") ;
	free(G) ;
	return(0) ;
	}
