/*
*  Program name:             g_seppou  ver. 1.0
*
*  Program function:         JKM grid-data seppou generation
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jun 7, 1995
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. window size (default:2)
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   JKM format (to stdout)
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#define	max(a, b)	(((a) > (b)) ? (a) : (b))
#define	min(a, b)	(((a) < (b)) ? (a) : (b))

#include "jkm_lib.c"

float	*G2 ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, ix, iy, nwin ;
	long	n ;
	double	g1, g2, g3, g4 ;
	void	JKM_Read(), JKM_Write() ;
/* -------------------------------------------------------------------------- */
	if (argc > 1 && !strcmp(argv[1], "-h")) {
	  fprintf(stderr, " USAGE: g_seppou {window-size}\n") ;
	  return(0) ;
	  }
	if (argc > 1) nwin = atoi(argv[1]) ; else nwin = 1 ;
/* -------------------------------------------------------------------------- */
	while (1) {
	JKM_Read(0) ;
	if (strcmp(Cmnt, "NULL") == 0) break ;
	fprintf(stderr, " Grid read end\n") ;
	if ((G2 = (float *)calloc(Nx*Ny, sizeof(*G2))) == NULL) {
	  fprintf(stderr, " Can't alloc memory G2.\n") ;
	  return(-1) ;
	  }
	for (n = 0; n < Nx*Ny; n++) G2[n] = G[n] ;
/* -------------------------------------------------------------------------- */
	for (i = 0; i < nwin; i++) {
	  fprintf(stderr, "Iteration %d start.\n", i) ;
	  Gmin =  9999999.9 ;
	  Gmax = -9999999.9 ;
	  for (ix = 0; ix < Nx; ix++) for (iy = 0; iy < Ny; iy++) {
	    n = ix * Ny + iy ;
	    if (G[n] == Und) continue ;
	    g1 = g2 = g3 = g4 = G[n] ;
	    if (ix > 0 && ix < Nx-1) {
	      if (G[n-Ny]!=Und&&G[n+Ny]!=Und) g1 = (G[n-Ny] + G[n+Ny]) / 2.0 ;
	      if (iy > 0 && iy < Ny-1) {
		if (G[n-Ny-1]!=Und&&G[n+Ny+1]!=Und)
	          g2 = (G[n-Ny-1] + G[n+Ny+1]) / 2.0 ;
		if (G[n+Ny-1]!=Und&&G[n-Ny+1]!=Und)
	          g3 = (G[n+Ny-1] + G[n-Ny+1]) / 2.0 ;
	        }
	      }
	    if (iy > 0 && iy < Ny-1) {
	      if (G[n-1]!=Und&&G[n+1]!=Und) g4 = (G[n-1] + G[n+1]) / 2.0 ;
	      }
	    G2[n] = max(g1, max(g2, max(g3, g4))) ;
	    Gmin = min(Gmin, G2[n]) ;
	    Gmax = max(Gmax, G2[n]) ;
	    }
	  for (n = 0; n < Nx*Ny; n++) G[n] = G2[n] ;
	  }
	JKM_Write() ;
	free(G) ;
	free(G2) ;
	}
	return(0) ;
	}
