/*
*  Program name:             g_smooth  ver. 1.0
*
*  Program function:         JKM grid-data smoothing by moving window
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Nov 13, 1993
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. smoothing factor ( 0.0 - 1.0 default:1.0 )
*                            2. smoothing window size
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   JKM format (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "jkm_lib.c"

float	Gmin2, Gmax2, *G2 ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ix, iy, jx, jy, kx, ky, nwin ;
	long	n, m ;
	double	atof(), s, w ;
	void	JKM_Read(), JKM_Write() ;
/* -------------------------------------------------------------------------- */
	if (argc > 1 && !strcmp(argv[1], "-h")) {
		fprintf(stderr, " USAGE: g_smooth {factor window-size}\n") ;
		return(0) ;
		}
	if (argc > 1) s    = atof(argv[1]) ; else s    = 1.0 ;
	if (argc > 2) nwin = atoi(argv[2]) ; else nwin = 1 ;
/* -------------------------------------------------------------------------- */
	while (1) {
	JKM_Read(0) ;
	if (strcmp(Cmnt, "NULL") == 0) break ;
	fprintf(stderr, " Grid read end\n") ;
	if ((G2 = (float *)calloc(Nx*Ny, sizeof(*G2))) == NULL) {
		fprintf(stderr, " Can't alloc memory G2.\n") ;
		return(-1) ;
		}
/* -------------------------------------------------------------------------- */
	Gmin2 =  9999999.9 ;
	Gmax2 = -9999999.9 ;
	for (ix = 0; ix < Nx; ix++) for (iy = 0; iy < Ny; iy++) {
		n = ix * Ny + iy ;
		if (G[n] == Und) {
			G2[n] = Und ;
			continue ;
			}
		m = 0 ;
		w = 0.0 ;
/*		nwin = max(1, (ix+iy)/150) ;	*/
		for (jx = -nwin ; jx <= nwin; jx++) {
			for (jy = -nwin ; jy <= nwin; jy++) {
				kx = ix + jx ;
				ky = iy + jy ;
				if (kx<0||kx>=Nx||ky<0||ky>=Ny) continue ;
				n = kx * Ny + ky ;
				if (G[n] != Und) {
					m++ ;
					w = w + G[n] ;
					}
				}
			}
		w = w / (double)m ;
		n = ix * Ny + iy ;
		G2[n] = w * s + G[n] * (1.0 - s) ;
		Gmin2 = min(Gmin2, G2[n]) ;
		Gmax2 = max(Gmax2, G2[n]) ;
		}
	Gmin = Gmin2 ;
	Gmax = Gmax2 ;
	for (n = 0; n < Nx*Ny; n++) G[n] = G2[n] ;
	JKM_Write() ;
	free(G) ;
	free(G2) ;
	}
	return(0) ;
	}
