/*
*  Program name:             ks476_g  ver. 1.0
*
*  Program function:         Create JKM grid file from KS-476
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jan 24, 1995
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. lower-left  corner longitude (unit:deg.min)
*                            2. lower-left  corner latitude  (unit:deg.min)
*                            3. upper-right corner longitude (unit:deg.min)
*                            4. upper-right corner latitude  (unit:deg.min)
*
*  Input:                    KS-476 file ( from standard input )
*
*  Output:                   JKM file ( to standard output
*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>

char	cmnt[100] ;
int	nx, ny ;
double	xstr, ystr, dx, dy ;
int	und, gmin, gmax, *g ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	minlon, minlat, maxlon, maxlat, deg, min, atof() ;
	int	latmin, latmax, lonmin, lonmax ;
	int	lat1, lon1, lat2, lon2, lat3, lon3, lat1o, lon1o ;
	int	ix, iy, jx, jy, l, atoi(), total ;
	char	tx[5] ;
	void	grdwtr_i() ;

	if ( argc < 5 ) {
		fprintf(stderr,
			" Usage : ks476_g minlon minlat maxlon maxlat\n") ;
		return(0) ;
		}

	minlon = atof(argv[1]) ;
	minlat = atof(argv[2]) ;
	maxlon = atof(argv[3]) ;
	maxlat = atof(argv[4]) ;

	deg = (int)(minlat) ;
	min = (minlat - deg) * 100.0 ;
	latmin = deg * 1200.0 + min * 20.0 + 0.5 ;
	ystr = (double)latmin * 3.0 ;

	deg = (int)(minlon) ;
	min = (minlon - deg) * 100.0 ;
	lonmin = deg *  800.0 + min * 80.0 / 6.0 + 0.5 ;
	xstr = (double)lonmin * 4.5 ;

	deg = (int)(maxlat) ;
	min = (maxlat - deg) * 100.0 ;
	latmax = deg * 1200.0 + min * 20.0 + 0.5 ;

	deg = (int)(maxlon) ;
	min = (maxlon - deg) * 100.0 ;
	lonmax = deg *  800.0 + min * 80.0 / 6.0 + 0.5 ;

	nx = lonmax - lonmin + 1 ;
	ny = latmax - latmin + 1 ;
	dx = 4.5 ;
	dy = 3.0 ;
	und = 99999 ;
	gmin = 1 ;
	gmax = 15 ;

	fprintf(stderr, " %5d%5d%11.2lf%11.2lf%9.3lf%9.3lf%10d%10d%10d\n",
		nx, ny, xstr, ystr, dx, dy, und, gmin, gmax) ;

	if ( (g = (int *)calloc(nx*ny, sizeof(*g))) == NULL ) {
		fprintf(stderr, " Can't alloc memory g.\n") ;
		return(-1) ;
		}

	lat1o = 0 ;
	lon1o = 0 ;
	total = 0 ;
	while ( scanf("%2d%2d%1d%1d%1d%1d%1d%1d",
		&lat1, &lon1, &lat2, &lon2, &lat3, &lon3, &iy, &ix) == 8 ) {
		if ( lat1 != lat1o || lon1 != lon1o )
			fprintf(stderr, " MESH = %d%d\n", lat1, lon1) ;
/*		if ( lat1 * 800 - latmin > ny ) break ; */
		lat1o = lat1 ;
		lon1o = lon1 ;
		jx = (lon1 + 100) * 800 + lon2 * 100 + lon3 * 10 + ix - lonmin ;
		jy =  lat1        * 800 + lat2 * 100 + lat3 * 10 + iy - latmin ;
		tx[0] = getchar() ;
		tx[1] = getchar() ;
		tx[2] = getchar() ;
		tx[3] = getchar() ;
		tx[4] = getchar() ;
		for ( ix=0; ix<15; ix++ ) getchar() ;
		if ( jx<0 || jx>=nx || jy<0 || jy>=ny ) continue ;
		l = jx * ny + jy ;
		g[l] = atoi(tx) ;
		total++ ;
		}

	fprintf(stderr, " Total = %d\n", total) ;

	strcpy(cmnt, "Generate from the KS-476 file developed by GSI\0") ;
	grdwtr_i() ;
	return(0) ;
	}

/* -------------------------------------------------------------------------- */
void	grdwtr_i()
{
	int	n ;

	for ( n = 0; n < nx*ny; n++ ) {
		if ( g[n] == und ) continue ;
		if ( gmin > g[n] ) gmin = g[n] ;
		if ( gmax < g[n] ) gmax = g[n] ;
		}

	printf("%s\n", cmnt) ;
	printf("%5d%5d%11.2lf%11.2lf%9.3lf%9.3lf%10d%10d%10d\n",
		nx, ny, xstr, ystr, dx, dy, und, gmin, gmax) ;

	for ( n = 0; n < nx*ny; n++ ) {
		if ( g[n] == und ) printf("%10d", und) ;
		else printf("%10d", g[n]) ;
		if ( n%8 == 7 ) putchar('\n') ;
		}

	if ( n%8 != 0 ) putchar('\n') ;

	}
