/*
*  Program name:             mem_g  ver. 1.0
*
*  Program function:         JKM data making from 50m topographic mesh data by GSI
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            May 1, 1995
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. Nx
*                            2. Ny
*                            3. Xstr
*                            4. Ystr
*
*  Input:                    GSI 50m MEM mesh data (from stdin)
*
*  Output:                   JKM mesh data (to stdout)
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#define	max(a, b)	(((a) > (b)) ? (a) : (b))

#include "jkm_lib.c"

main (argc, argv)
int	argc ;
char	*argv[] ;
{
	int	lat1, lon1, lat2, lon2, ix, iy, ix0, iy0, x, y, n ;
	int	atoi() ;
	double	atof() ;
	char	line[1050], tx[7] ;

	if (argc >  1) Nx   = atoi(argv[1]) ; else Nx   = 1400 ;
	if (argc >  2) Ny   = atoi(argv[2]) ; else Ny   = 1800 ;
	if (argc >  3) Xstr = atof(argv[3]) ; else Xstr = 484650.0 ;
	if (argc >  4) Ystr = atof(argv[4]) ; else Ystr = 122700.0 ;

	Dx = 2.25 ;
	Dy = 1.5 ;
	Und = -999.9 ;

	strcpy(Cmnt, "Grid data generated from 50m-mesh by GSI.") ;
	if ((G = (float *)calloc(Nx*Ny, sizeof(*G))) == NULL) {
		fprintf(stderr, " Can't alloc memory G.\n") ;
		return(-1) ;
		}

	ix0 = Xstr / Dx ;
	iy0 = Ystr / Dy ;
	Xstr = (double)ix0 * Dx + Dx / 2.0 ;
	Ystr = (double)iy0 * Dy + Dy / 2.0 ;

	for (n = 0; n < Nx*Ny; n++) G[n] = Und ;

	while ( fgets(line, 1050, stdin) != NULL ) {
		if ( line[1008] == ' ' || line[1008] == '\0' ) continue ;
		sscanf(line, "%2d%2d%1d%1d%3d", &lat1, &lon1, &lat2, &lon2, &y) ;
		iy = (200 - y) + (lat1 * 8 + lat2) * 200 - iy0 ;
		if ( iy < 0 || iy >= Ny ) continue ;
		for ( x = 0; x < 200; x++ ) {
			ix = x + (800 + lon1 * 8 + lon2) * 200 - ix0 ;
			if ( ix < 0 || ix >= Nx ) continue ;
			tx[0] = line[x * 5 +  9] ;
			tx[1] = line[x * 5 + 10] ;
			tx[2] = line[x * 5 + 11] ;
			tx[3] = line[x * 5 + 12] ;
			tx[4] = line[x * 5 + 13] ;
			tx[5] = '.' ;
			tx[6] = '0' ;
			n = ix * Ny + iy ;
			G[n] = atof(tx) / 10.0 ;
			if (G[n] >= 7000.0) {
				fprintf(stderr, "%d %d %s %f\n", x, y, tx, G[n]) ;
				fprintf(stderr, "%s\n", line) ;
				}
			if (G[n] < 0.0) G[n] = Und ;
			}
		}

	JKM_Write() ;

	return(0) ;

	}
