/*
*  Program name:             ppm_cal  ver. 2.0
*
*  Program function:         Calculation between PPM and PGM
*                            OutPPM = InPGM/255 * InPPM * scale + shift
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 15, 1990
*
*  Date of revision:         June 11, 1995  Change to PPM/PGM by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -sc #   (scale value default: 1.0)
*                            -sh #   (shift value default: 0.0)
*                            -update (update to latest data)
*                            -pic    (read/write pictro format)
*
*  Input:                    PGM (1st) and PPM (2nd) file (from stdin)
*
*  Output:                   PPM file (to stdout)
*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#define	P3	0
#define	P5	1
#define	P6	2
#define	PIC	3
#define	PIC2	4

#define	max(a, b)	(((a) > (b)) ? (a) : (b))
#define	min(a, b)	(((a) < (b)) ? (a) : (b))

long	Xdots, Ydots, Resol, Size= -1, upd=0 ;
unsigned char	*R, *G, *B ;

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	long	i, nx, ny, nf, x, y, Pic=0, kind ;
	long	n ;
	float	scale=1.0, shift=0.0 ;
	double	atof() ;
	short	r, g, b ;
	char	tx[70] ;
	void	update() ;
/* -------------------------------------------------------------------------- */
	for (i = 1; i < argc; i++) {
	  if (!strcmp(argv[i], "-h")) {
	    fprintf(stderr, " USAGE: ppm_cal {-sc # -sh # -pic}\n") ;
	    return(0) ;
	    }
	  else if (!strcmp(argv[i], "-sc"))     scale = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-sh"))     shift = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-update")) upd = 1 ;
	  else if (!strcmp(argv[i], "-pic"))    Pic = 1 ;
	  else if (!strcmp(argv[i], "-pic2"))   Pic = 1 ;
	  else if (!strcmp(argv[i], "-spic"))   Pic = 1 ;
	  else if (!strcmp(argv[i], "-spic2"))  Pic = 1 ;
	  }
	scale = scale / 255.0 ;
/* -------------------------------------------------------------------------- */
	for (nf = 1; nf < 1000; nf++) {

	  for (n = 0; n < 3; n++) {
	    if (fgets(tx, 70, stdin) == NULL) goto FIN ;
	    if (tx[0] == '#') n-- ;
	    else if (n == 0) {
	      if (strncmp(tx, "P3", 2) == 0)
	        kind = P3 ;
	      else if (strncmp(tx, "P5", 2) == 0)
	        kind = P5 ;
	      else if (strncmp(tx, "P6", 2) == 0)
	        kind = P6 ;
	      else if (sscanf(tx, "%d %d %d %d", &Resol,&Size,&nx,&ny) == 4) {
	        if (Size == 0) {
	          kind = ny ;
	          ny = nx ;
	          nx = kind ;
	          kind = PIC ;
	          }
	        else {
	          kind = PIC2 ;
	          }
	        break ;
	        }
	      else {
	        fprintf(stderr, "First image data error.\n") ;
	        return(-1) ;
	        }
	      }
	    else if (n == 1 && sscanf(tx, "%d %d", &nx, &ny) != 2) {
	      fprintf(stderr, " Can't read P5/P6 header.\n") ;
	      return(-1) ;
	      }
	    }
	  fprintf(stderr, " File %d (kind %d) read start %d %d.\n",
	    nf, kind, nx, ny) ;
/* -------------------------------------------------------------------------- */
	  if (nf == 1) {
	    Xdots = nx ;
	    Ydots = ny ;
	    if ((R = (unsigned char *)calloc(Xdots*Ydots,sizeof(*R))) == NULL) {
	      fprintf(stderr, " Can't alloc memory R.\n") ;
	      return(-1) ;
	      }
	    if ((G = (unsigned char *)calloc(Xdots*Ydots,sizeof(*G))) == NULL) {
	      fprintf(stderr, " Can't alloc memory G.\n") ;
	      return(-1) ;
	      }
	    if ((B = (unsigned char *)calloc(Xdots*Ydots,sizeof(*B))) == NULL) {
	      fprintf(stderr, " Can't alloc memory B.\n") ;
	      return(-1) ;
	      }
	    }
/* -------------------------------------------------------------------------- */
	  if (kind == P3) {
	    for (y = ny-1; y > -1; y--) for (x = 0; x < nx; x++) {
	      n = x * Ydots + y ;
	      if (scanf("%d %d %d", &r, &g, &b) != 3) {
	        fprintf(stderr, " P3 file reading error.\n") ;
	        return(-1) ;
	        }
	      update(nf, n, r, g, b, scale, shift) ;
	      }
	    }
	  else if (kind == P5) {
	    for (y = ny-1; y > -1; y--) for (x = 0; x < nx; x++) {
	      n = x * Ydots + y ;
	      r = getchar() ;
	      g = r ;
	      b = r ;
	      update(nf, n, r, g, b, scale, shift) ;
	      }
	    }
	  else if (kind == P6) {
	    for (y = ny-1; y > -1; y--) for (x = 0; x < nx; x++) {
	      n = x * Ydots + y ;
	      r = getchar() ;
	      g = getchar() ;
	      b = getchar() ;
	      update(nf, n, r, g, b, scale, shift) ;
	      }
	    }
	  else if (kind == PIC) {
	    for(x = 0; x < nx; x++) for(y = 0; y < ny; y++) {
	      n = x * Ydots + y ;
	      r = getchar() ;
	      g = getchar() ;
	      b = getchar() ;
	      update(nf, n, r, g, b, scale, shift) ;
	      }
	    }
	  else if (kind == PIC2) {
	    for (y = ny-1; y > -1; y--) for (x = 0; x < nx; x++) {
	      n = x * Ydots + y ;
	      r = getchar() ;
	      g = getchar() ;
	      b = getchar() ;
	      update(nf, n, r, g, b, scale, shift) ;
	      }
	    }
/*	  return(0) ;*/
	  }
/* -------------------------------------------------------------------------- */
FIN:
	if (Pic == 0) {
	  printf("P6\n") ;
	  printf("%5ld %5ld\n", Xdots, Ydots) ;
	  printf("255\n") ;
	  for (y = Ydots-1; y > -1; y--) for (x = 0; x < Xdots; x++) {
	    n = x * Ydots + y ;
	    putchar(R[n]) ;
	    putchar(G[n]) ;
	    putchar(B[n]) ;
	    }
	  }
	else if (Size == 0) {
	  printf("%d %d %d %d\n", Resol, 0, Ydots, Xdots) ;
	  for(x = 0; x < Xdots; x++) for(y = 0; y < Ydots; y++) {
	    n = x * Ydots + y ;
	    putchar(R[n]) ;
	    putchar(G[n]) ;
	    putchar(B[n]) ;
	    }
	  }
	else {
	  printf("%d %d %d %d\n", Resol, 1, Xdots, Ydots) ;
	  for (y = Ydots-1; y > -1; y--) for (x = 0; x < Xdots; x++) {
	    n = x * Ydots + y ;
	    putchar(R[n]) ;
	    putchar(G[n]) ;
	    putchar(B[n]) ;
	    }
	  }
	free(R) ;
	free(G) ;
	free(B) ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	update(nf, n, r, g, b, scale, shift)
int	nf, n, r, g, b ;
float	scale, shift ;
{
	if (nf > 1) {
	  if (r != 255 || g != 255 || b != 255) {
	    if (upd == 0) {
	      r = (float)r * (float)R[n] * scale + shift + 0.5 ;
	      g = (float)g * (float)G[n] * scale + shift + 0.5 ;
	      b = (float)b * (float)B[n] * scale + shift + 0.5 ;
	      R[n] = min(255, max(0, r)) ;
	      G[n] = min(255, max(0, g)) ;
	      B[n] = min(255, max(0, b)) ;
	      }
	    else {
	      R[n] = r ;
	      G[n] = g ;
	      B[n] = b ;
	      }
	    }
	  }
	else {
	  R[n] = min(255, max(0, r)) ;
	  G[n] = min(255, max(0, g)) ;
	  B[n] = min(255, max(0, b)) ;
 	  }
	}
